@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 from __future__ import print_function
+from distutils.version import LooseVersion
 # pylint: disable=W0141
 
 import sys
@@ -12,21 +13,22 @@ from pandas.compat import(StringIO, lzip, range, map, zip, reduce, u,
                           OrderedDict)
 from pandas.util.terminal import get_terminal_size
 from pandas.core.config import get_option, set_option
+from pandas.io.common import _get_handle, UnicodeWriter, _expand_user
 import pandas.core.common as com
 import pandas.lib as lib
 from pandas.tslib import iNaT, Timestamp, Timedelta, format_array_from_datetime
 from pandas.tseries.index import DatetimeIndex
 from pandas.tseries.period import PeriodIndex
+import pandas as pd
 import numpy as np
 
 import itertools
 import csv
+import warnings
 
-docstring_to_string = """
-     Parameters
-     ----------
-     frame : DataFrame
-         object to render
+common_docstring = """
+    Parameters
+    ----------
     buf : StringIO-like, optional
         buffer to write to
     columns : sequence, optional
@@ -49,28 +51,29 @@ docstring_to_string = """
     sparsify : bool, optional
         Set to False for a DataFrame with a hierarchical index to print every
         multiindex key at each row, default True
+    index_names : bool, optional
+        Prints the names of the indexes, default True"""
+
+justify_docstring  = """
     justify : {'left', 'right'}, default None
         Left or right-justify the column labels. If None uses the option from
         the print configuration (controlled by set_option), 'right' out
-        of the box.
-    index_names : bool, optional
-        Prints the names of the indexes, default True
-    force_unicode : bool, default False
-        Always return a unicode result. Deprecated in v0.10.0 as string
-        formatting is now rendered to unicode by default.
+        of the box."""
+
+return_docstring = """
 
     Returns
     -------
     formatted : string (or unicode, depending on data and options)"""
 
+docstring_to_string = common_docstring + justify_docstring + return_docstring
 
 class CategoricalFormatter(object):
 
     def __init__(self, categorical, buf=None, length=True,
-                 na_rep='NaN', name=False, footer=True):
+                 na_rep='NaN', footer=True):
         self.categorical = categorical
         self.buf = buf if buf is not None else StringIO(u(""))
-        self.name = name
         self.na_rep = na_rep
         self.length = length
         self.footer = footer
@@ -78,12 +81,6 @@ class CategoricalFormatter(object):
     def _get_footer(self):
         footer = ''
 
-        if self.name:
-            name = com.pprint_thing(self.categorical.name,
-                                    escape_chars=('\t', '\r', '\n'))
-            footer += ('Name: %s' % name if self.categorical.name is not None
-                       else '')
-
         if self.length:
             if footer:
                 footer += ', '
@@ -143,6 +140,7 @@ class SeriesFormatter(object):
             float_format = get_option("display.float_format")
         self.float_format = float_format
         self.dtype = dtype
+        self.adj = _get_adjustment()
 
         self._chk_truncate()
 
@@ -191,9 +189,8 @@ class SeriesFormatter(object):
                 footer += 'dtype: %s' % com.pprint_thing(name)
 
         # level infos are added to the end and in a new line, like it is done for Categoricals
-        # Only added when we request a name
-        if name and com.is_categorical_dtype(self.tr_series.dtype):
-            level_info = self.tr_series.values._repr_categories_info()
+        if com.is_categorical_dtype(self.tr_series.dtype):
+            level_info = self.tr_series._values._repr_categories_info()
             if footer:
                 footer += "\n"
             footer += level_info
@@ -213,7 +210,7 @@ class SeriesFormatter(object):
         return fmt_index, have_header
 
     def _get_formatted_values(self):
-        return format_array(self.tr_series.get_values(), None,
+        return format_array(self.tr_series._values, None,
                             float_format=self.float_format,
                             na_rep=self.na_rep)
 
@@ -227,22 +224,24 @@ class SeriesFormatter(object):
         fmt_index, have_header = self._get_formatted_index()
         fmt_values = self._get_formatted_values()
 
-        maxlen = max(len(x) for x in fmt_index)  # max index len
+        maxlen = max(self.adj.len(x) for x in fmt_index)  # max index len
         pad_space = min(maxlen, 60)
 
         if self.truncate_v:
             n_header_rows = 0
             row_num = self.tr_row_num
-            width = len(fmt_values[row_num-1])
+            width = self.adj.len(fmt_values[row_num-1])
             if width > 3:
                 dot_str = '...'
             else:
                 dot_str = '..'
-            dot_str = dot_str.center(width)
+            # Series uses mode=center because it has single value columns
+            # DataFrame uses mode=left
+            dot_str = self.adj.justify([dot_str], width, mode='center')[0]
             fmt_values.insert(row_num + n_header_rows, dot_str)
             fmt_index.insert(row_num + 1, '')
 
-        result = adjoin(3, *[fmt_index[1:], fmt_values])
+        result = self.adj.adjoin(3, *[fmt_index[1:], fmt_values])
 
         if self.header and have_header:
             result = fmt_index[0] + '\n' + result
@@ -253,19 +252,54 @@ class SeriesFormatter(object):
         return compat.text_type(u('').join(result))
 
 
-def _strlen_func():
-    if compat.PY3:  # pragma: no cover
-        _strlen = len
+class TextAdjustment(object):
+
+    def __init__(self):
+        self.encoding = get_option("display.encoding")
+
+    def len(self, text):
+        return compat.strlen(text, encoding=self.encoding)
+
+    def justify(self, texts, max_len, mode='right'):
+        return com._justify(texts, max_len, mode=mode)
+
+    def adjoin(self, space, *lists, **kwargs):
+        return com.adjoin(space, *lists, strlen=self.len,
+                          justfunc=self.justify, **kwargs)
+
+
+class EastAsianTextAdjustment(TextAdjustment):
+
+    def __init__(self):
+        super(EastAsianTextAdjustment, self).__init__()
+        if get_option("display.unicode.ambiguous_as_wide"):
+            self.ambiguous_width = 2
+        else:
+            self.ambiguous_width = 1
+
+    def len(self, text):
+        return compat.east_asian_len(text, encoding=self.encoding,
+                                     ambiguous_width=self.ambiguous_width)
+
+    def justify(self, texts, max_len, mode='right'):
+        # re-calculate padding space per str considering East Asian Width
+        def _get_pad(t):
+            return max_len - self.len(t) + len(t)
+
+        if mode == 'left':
+            return [x.ljust(_get_pad(x)) for x in texts]
+        elif mode == 'center':
+            return [x.center(_get_pad(x)) for x in texts]
+        else:
+            return [x.rjust(_get_pad(x)) for x in texts]
+
+
+def _get_adjustment():
+    use_east_asian_width = get_option("display.unicode.east_asian_width")
+    if use_east_asian_width:
+        return EastAsianTextAdjustment()
     else:
-        encoding = get_option("display.encoding")
-
-        def _strlen(x):
-            try:
-                return len(x.decode(encoding))
-            except UnicodeError:
-                return len(x)
-
-    return _strlen
+        return TextAdjustment()
 
 
 class TableFormatter(object):
@@ -301,7 +335,7 @@ class DataFrameFormatter(TableFormatter):
     """
 
     __doc__ = __doc__ if __doc__ else ''
-    __doc__ += docstring_to_string
+    __doc__ += common_docstring + justify_docstring + return_docstring
 
     def __init__(self, frame, buf=None, columns=None, col_space=None,
                  header=True, index=True, na_rep='NaN', formatters=None,
@@ -309,7 +343,7 @@ class DataFrameFormatter(TableFormatter):
                  index_names=True, line_width=None, max_rows=None,
                  max_cols=None, show_dimensions=False, **kwds):
         self.frame = frame
-        self.buf = buf if buf is not None else StringIO()
+        self.buf = _expand_user(buf) if buf is not None else StringIO()
         self.show_index_names = index_names
 
         if sparsify is None:
@@ -344,6 +378,7 @@ class DataFrameFormatter(TableFormatter):
             self.columns = frame.columns
 
         self._chk_truncate()
+        self.adj = _get_adjustment()
 
     def _chk_truncate(self):
         '''
@@ -420,7 +455,6 @@ class DataFrameFormatter(TableFormatter):
         """
         Render a DataFrame to a list of columns (as lists of strings).
         """
-        _strlen = _strlen_func()
         frame = self.tr_frame
 
         # may include levels names also
@@ -433,27 +467,23 @@ class DataFrameFormatter(TableFormatter):
             for i, c in enumerate(frame):
                 cheader = str_columns[i]
                 max_colwidth = max(self.col_space or 0,
-                                   *(_strlen(x) for x in cheader))
-
+                                   *(self.adj.len(x) for x in cheader))
                 fmt_values = self._format_col(i)
-
                 fmt_values = _make_fixed_width(fmt_values, self.justify,
-                                               minimum=max_colwidth)
+                                               minimum=max_colwidth,
+                                               adj=self.adj)
 
-                max_len = max(np.max([_strlen(x) for x in fmt_values]),
+                max_len = max(np.max([self.adj.len(x) for x in fmt_values]),
                               max_colwidth)
-                if self.justify == 'left':
-                    cheader = [x.ljust(max_len) for x in cheader]
-                else:
-                    cheader = [x.rjust(max_len) for x in cheader]
-
+                cheader = self.adj.justify(cheader, max_len, mode=self.justify)
                 stringified.append(cheader + fmt_values)
         else:
             stringified = []
             for i, c in enumerate(frame):
                 fmt_values = self._format_col(i)
                 fmt_values = _make_fixed_width(fmt_values, self.justify,
-                                               minimum=(self.col_space or 0))
+                                               minimum=(self.col_space or 0),
+                                               adj=self.adj)
 
                 stringified.append(fmt_values)
 
@@ -467,13 +497,13 @@ class DataFrameFormatter(TableFormatter):
 
         if truncate_h:
             col_num = self.tr_col_num
-            col_width = len(strcols[self.tr_size_col][0])  # infer from column header
+            col_width = self.adj.len(strcols[self.tr_size_col][0])  # infer from column header
             strcols.insert(self.tr_col_num + 1, ['...'.center(col_width)] * (len(str_index)))
         if truncate_v:
             n_header_rows = len(str_index) - len(frame)
             row_num = self.tr_row_num
             for ix, col in enumerate(strcols):
-                cwidth = len(strcols[ix][row_num])  # infer from above row
+                cwidth = self.adj.len(strcols[ix][row_num])  # infer from above row
                 is_dot_col = False
                 if truncate_h:
                     is_dot_col = ix == col_num + 1
@@ -483,13 +513,13 @@ class DataFrameFormatter(TableFormatter):
                     my_str = '..'
 
                 if ix == 0:
-                    dot_str = my_str.ljust(cwidth)
+                    dot_mode = 'left'
                 elif is_dot_col:
-                    cwidth = len(strcols[self.tr_size_col][0])
-                    dot_str = my_str.center(cwidth)
+                    cwidth = self.adj.len(strcols[self.tr_size_col][0])
+                    dot_mode = 'center'
                 else:
-                    dot_str = my_str.rjust(cwidth)
-
+                    dot_mode = 'right'
+                dot_str = self.adj.justify([my_str], cwidth, mode=dot_mode)[0]
                 strcols[ix].insert(row_num + n_header_rows, dot_str)
         return strcols
 
@@ -498,6 +528,7 @@ class DataFrameFormatter(TableFormatter):
         Render a DataFrame to a console-friendly tabular output.
         """
         from pandas import Series
+
         frame = self.frame
 
         if len(frame.columns) == 0 or len(frame.index) == 0:
@@ -509,11 +540,11 @@ class DataFrameFormatter(TableFormatter):
         else:
             strcols = self._to_str_columns()
             if self.line_width is None:  # no need to wrap around just print the whole frame
-                text = adjoin(1, *strcols)
+                text = self.adj.adjoin(1, *strcols)
             elif not isinstance(self.max_cols, int) or self.max_cols > 0:  # need to wrap around
                 text = self._join_multiline(*strcols)
             else:  # max_cols == 0. Try to fit frame to terminal
-                text = adjoin(1, *strcols).split('\n')
+                text = self.adj.adjoin(1, *strcols).split('\n')
                 row_lens = Series(text).apply(len)
                 max_len_col_ix = np.argmax(row_lens)
                 max_len = row_lens[max_len_col_ix]
@@ -541,7 +572,7 @@ class DataFrameFormatter(TableFormatter):
                 # and then generate string representation
                 self._chk_truncate()
                 strcols = self._to_str_columns()
-                text = adjoin(1, *strcols)
+                text = self.adj.adjoin(1, *strcols)
 
         self.buf.writelines(text)
 
@@ -555,9 +586,9 @@ class DataFrameFormatter(TableFormatter):
         strcols = list(strcols)
         if self.index:
             idx = strcols.pop(0)
-            lwidth -= np.array([len(x) for x in idx]).max() + adjoin_width
+            lwidth -= np.array([self.adj.len(x) for x in idx]).max() + adjoin_width
 
-        col_widths = [np.array([len(x) for x in col]).max()
+        col_widths = [np.array([self.adj.len(x) for x in col]).max()
                       if len(col) > 0 else 0
                       for col in strcols]
         col_bins = _binify(col_widths, lwidth)
@@ -578,8 +609,7 @@ class DataFrameFormatter(TableFormatter):
                     row.append([' \\'] + ['  '] * (nrows - 1))
                 else:
                     row.append([' '] * nrows)
-
-            str_lst.append(adjoin(adjoin_width, *row))
+            str_lst.append(self.adj.adjoin(adjoin_width, *row))
             st = ed
         return '\n\n'.join(str_lst)
 
@@ -589,7 +619,6 @@ class DataFrameFormatter(TableFormatter):
         """
         self.escape = self.kwds.get('escape', True)
 
-        # TODO: column_format is not settable in df.to_latex
         def get_col_type(dtype):
             if issubclass(dtype.type, np.number):
                 return 'r'
@@ -609,11 +638,13 @@ class DataFrameFormatter(TableFormatter):
         if self.index and isinstance(self.frame.index, MultiIndex):
             clevels = self.frame.columns.nlevels
             strcols.pop(0)
-            name = any(self.frame.columns.names)
+            name = any(self.frame.index.names)
             for i, lev in enumerate(self.frame.index.levels):
-                lev2 = lev.format(name=name)
+                lev2 = lev.format()
                 blank = ' ' * len(lev2[0])
                 lev3 = [blank] * clevels
+                if name:
+                    lev3.append(lev.name)
                 for level_idx, group in itertools.groupby(
                         self.frame.index.labels[i]):
                     count = len(list(group))
@@ -621,7 +652,7 @@ class DataFrameFormatter(TableFormatter):
                 strcols.insert(i, lev3)
 
         if column_format is None:
-            dtypes = self.frame.dtypes.values
+            dtypes = self.frame.dtypes._values
             column_format = ''.join(map(get_col_type, dtypes))
             if self.index:
                 index_format = 'l' * self.frame.index.nlevels
@@ -640,8 +671,10 @@ class DataFrameFormatter(TableFormatter):
                 buf.write('\\toprule\n')
 
             nlevels = frame.columns.nlevels
+            if any(frame.index.names):
+                nlevels += 1
             for i, row in enumerate(zip(*strcols)):
-                if i == nlevels:
+                if i == nlevels and self.header:
                     buf.write('\\midrule\n')  # End of header
                     if longtable:
                         buf.write('\\endhead\n')
@@ -687,18 +720,25 @@ class DataFrameFormatter(TableFormatter):
         frame = self.tr_frame
         formatter = self._get_formatter(i)
         return format_array(
-            (frame.iloc[:, i]).get_values(),
+            frame.iloc[:, i]._values,
             formatter, float_format=self.float_format, na_rep=self.na_rep,
             space=self.col_space
         )
 
-    def to_html(self, classes=None):
+    def to_html(self, classes=None, notebook=False):
         """
         Render a DataFrame to a html table.
+
+        Parameters
+        ----------
+        notebook : {True, False}, optional, default False
+            Whether the generated HTML is for IPython Notebook.
+
         """
         html_renderer = HTMLFormatter(self, classes=classes,
                                       max_rows=self.max_rows,
-                                      max_cols=self.max_cols)
+                                      max_cols=self.max_cols,
+                                      notebook=notebook)
         if hasattr(self.buf, 'write'):
             html_renderer.write_result(self.buf)
         elif isinstance(self.buf, compat.string_types):
@@ -719,7 +759,7 @@ class DataFrameFormatter(TableFormatter):
         if isinstance(columns, MultiIndex):
             fmt_columns = columns.format(sparsify=False, adjoin=False)
             fmt_columns = lzip(*fmt_columns)
-            dtypes = self.frame.dtypes.values
+            dtypes = self.frame.dtypes._values
 
             # if we have a Float level, they don't use leading space at all
             restrict_formatting = any([l.is_floating for l in columns.levels])
@@ -776,11 +816,12 @@ class DataFrameFormatter(TableFormatter):
                                      formatter=fmt)
         else:
             fmt_index = [index.format(name=show_index_names, formatter=fmt)]
-        fmt_index = [tuple(_make_fixed_width(
-            list(x), justify='left', minimum=(self.col_space or 0)))
-            for x in fmt_index]
+        fmt_index = [tuple(_make_fixed_width(list(x), justify='left',
+                                             minimum=(self.col_space or 0),
+                                             adj=self.adj))
+                     for x in fmt_index]
 
-        adjoined = adjoin(1, *fmt_index).split('\n')
+        adjoined = self.adj.adjoin(1, *fmt_index).split('\n')
 
         # empty space for columns
         if show_col_names:
@@ -808,7 +849,8 @@ class HTMLFormatter(TableFormatter):
 
     indent_delta = 2
 
-    def __init__(self, formatter, classes=None, max_rows=None, max_cols=None):
+    def __init__(self, formatter, classes=None, max_rows=None, max_cols=None,
+                 notebook=False):
         self.fmt = formatter
         self.classes = classes
 
@@ -823,6 +865,7 @@ class HTMLFormatter(TableFormatter):
         self.show_dimensions = self.fmt.show_dimensions
         self.is_truncated = (self.max_rows < len(self.fmt.frame) or
                              self.max_cols < len(self.fmt.columns))
+        self.notebook = notebook
 
     def write(self, s, indent=0):
         rs = com.pprint_thing(s)
@@ -890,6 +933,17 @@ class HTMLFormatter(TableFormatter):
                                       'not %s') % type(self.classes))
             _classes.extend(self.classes)
 
+        if self.notebook:
+            div_style = ''
+            try:
+                import IPython
+                if IPython.__version__ < LooseVersion('3.0.0'):
+                    div_style = ' style="max-width:1500px;overflow:auto;"'
+            except ImportError:
+                pass
+
+            self.write('<div{0}>'.format(div_style))
+
         self.write('<table border="1" class="%s">' % ' '.join(_classes),
                    indent)
 
@@ -902,6 +956,10 @@ class HTMLFormatter(TableFormatter):
             by = chr(215) if compat.PY3 else unichr(215)  # ×
             self.write(u('<p>%d rows %s %d columns</p>') %
                        (len(frame), by, len(frame.columns)))
+
+        if self.notebook:
+            self.write('</div>')
+
         _put_lines(buf, self.elements)
 
     def _write_header(self, indent):
@@ -1019,7 +1077,7 @@ class HTMLFormatter(TableFormatter):
             self.write_tr(col_row, indent, self.indent_delta, header=True,
                           align=align)
 
-        if self.fmt.has_index_names:
+        if self.fmt.has_index_names and self.fmt.index:
             row = [
                 x if x is not None else '' for x in self.frame.index.names
             ] + [''] * min(len(self.columns), self.max_cols)
@@ -1203,18 +1261,22 @@ class CSVFormatter(object):
 
     def __init__(self, obj, path_or_buf=None, sep=",", na_rep='', float_format=None,
                  cols=None, header=True, index=True, index_label=None,
-                 mode='w', nanRep=None, encoding=None, quoting=None,
+                 mode='w', nanRep=None, encoding=None, compression=None, quoting=None,
                  line_terminator='\n', chunksize=None, engine=None,
                  tupleize_cols=False, quotechar='"', date_format=None,
                  doublequote=True, escapechar=None, decimal='.'):
 
-        self.engine = engine  # remove for 0.13
+        if engine is not None:
+            warnings.warn("'engine' keyword is deprecated and "
+                          "will be removed in a future version",
+                          FutureWarning, stacklevel=3)
+        self.engine = engine  # remove for 0.18
         self.obj = obj
 
         if path_or_buf is None:
             path_or_buf = StringIO()
 
-        self.path_or_buf = path_or_buf
+        self.path_or_buf = _expand_user(path_or_buf)
         self.sep = sep
         self.na_rep = na_rep
         self.float_format = float_format
@@ -1225,6 +1287,7 @@ class CSVFormatter(object):
         self.index_label = index_label
         self.mode = mode
         self.encoding = encoding
+        self.compression = compression
 
         if quoting is None:
             quoting = csv.QUOTE_MINIMAL
@@ -1264,7 +1327,7 @@ class CSVFormatter(object):
                                             date_format=date_format,
                                             quoting=self.quoting)
             else:
-                cols = np.asarray(list(cols))
+                cols = list(cols)
             self.obj = self.obj.loc[:, cols]
 
         # update columns to include possible multiplicity of dupes
@@ -1276,7 +1339,7 @@ class CSVFormatter(object):
                                         date_format=date_format,
                                         quoting=self.quoting)
         else:
-            cols = np.asarray(list(cols))
+            cols = list(cols)
 
         # save it
         self.cols = cols
@@ -1383,7 +1446,7 @@ class CSVFormatter(object):
 
         series = {}
         for k, v in compat.iteritems(values._series):
-            series[k] = v.values
+            series[k] = v._values
 
         nlevels = getattr(data_index, 'nlevels', 1)
         for j, idx in enumerate(data_index):
@@ -1413,8 +1476,9 @@ class CSVFormatter(object):
             f = self.path_or_buf
             close = False
         else:
-            f = com._get_handle(self.path_or_buf, self.mode,
-                                encoding=self.encoding)
+            f = _get_handle(self.path_or_buf, self.mode,
+                                encoding=self.encoding,
+                                compression=self.compression)
             close = True
 
         try:
@@ -1425,7 +1489,7 @@ class CSVFormatter(object):
                                  quotechar=self.quotechar)
             if self.encoding is not None:
                 writer_kwargs['encoding'] = self.encoding
-                self.writer = com.UnicodeWriter(f, **writer_kwargs)
+                self.writer = UnicodeWriter(f, **writer_kwargs)
             else:
                 self.writer = csv.writer(f, **writer_kwargs)
 
@@ -1627,12 +1691,12 @@ class ExcelFormatter(object):
     def __init__(self, df, na_rep='', float_format=None, cols=None,
                  header=True, index=True, index_label=None, merge_cells=False,
                  inf_rep='inf'):
-        self.df = df
         self.rowcounter = 0
         self.na_rep = na_rep
-        self.columns = cols
-        if cols is None:
-            self.columns = df.columns
+        self.df = df
+        if cols is not None:
+            self.df = df.loc[:, cols]
+        self.columns = self.df.columns
         self.float_format = float_format
         self.index = index
         self.index_label = index_label
@@ -1644,21 +1708,28 @@ class ExcelFormatter(object):
         if lib.checknull(val):
             val = self.na_rep
         elif com.is_float(val):
-            if np.isposinf(val):
+            if lib.isposinf_scalar(val):
                 val = self.inf_rep
-            elif np.isneginf(val):
+            elif lib.isneginf_scalar(val):
                 val = '-%s' % self.inf_rep
             elif self.float_format is not None:
                 val = float(self.float_format % val)
         return val
 
     def _format_header_mi(self):
+
+        if self.columns.nlevels > 1:
+            if not self.index:
+                raise NotImplementedError("Writing to Excel with MultiIndex"
+                                          " columns and no index ('index'=False) "
+                                          "is not yet implemented.")
+
         has_aliases = isinstance(self.header, (tuple, list, np.ndarray, Index))
         if not(has_aliases or self.header):
             return
 
         columns = self.columns
-        level_strs = columns.format(sparsify=True, adjoin=False, names=False)
+        level_strs = columns.format(sparsify=self.merge_cells, adjoin=False, names=False)
         level_lengths = _get_level_lengths(level_strs)
         coloffset = 0
         lnum = 0
@@ -1762,18 +1833,14 @@ class ExcelFormatter(object):
             else:
                 index_label = self.df.index.names[0]
 
+            if isinstance(self.columns, MultiIndex):
+                self.rowcounter += 1
+
             if index_label and self.header is not False:
-                if self.merge_cells:
-                    yield ExcelCell(self.rowcounter,
-                                    0,
-                                    index_label,
-                                    header_style)
-                    self.rowcounter += 1
-                else:
-                    yield ExcelCell(self.rowcounter - 1,
-                                    0,
-                                    index_label,
-                                    header_style)
+                yield ExcelCell(self.rowcounter - 1,
+                                0,
+                                index_label,
+                                header_style)
 
             # write index_values
             index_values = self.df.index
@@ -1784,12 +1851,9 @@ class ExcelFormatter(object):
             for idx, idxval in enumerate(index_values):
                 yield ExcelCell(self.rowcounter + idx, 0, idxval, header_style)
 
-        # Get a frame that will account for any duplicates in the column names.
-        col_mapped_frame = self.df.loc[:, self.columns]
-
         # Write the body of the frame data series by series.
         for colidx in range(len(self.columns)):
-            series = col_mapped_frame.iloc[:, colidx]
+            series = self.df.iloc[:, colidx]
             for i, val in enumerate(series):
                 yield ExcelCell(self.rowcounter + i, colidx + coloffset, val)
 
@@ -1807,19 +1871,22 @@ class ExcelFormatter(object):
                                                (list, tuple, np.ndarray, Index)):
                 index_labels = self.index_label
 
+            # MultiIndex columns require an extra row
+            # with index names (blank if None) for
+            # unambigous round-trip, unless not merging,
+            # in which case the names all go on one row Issue #11328
+            if isinstance(self.columns, MultiIndex) and self.merge_cells:
+                self.rowcounter += 1
+
             # if index labels are not empty go ahead and dump
             if (any(x is not None for x in index_labels)
                     and self.header is not False):
 
-                if not self.merge_cells:
-                    self.rowcounter -= 1
-
                 for cidx, name in enumerate(index_labels):
-                    yield ExcelCell(self.rowcounter,
+                    yield ExcelCell(self.rowcounter - 1,
                                     cidx,
                                     name,
                                     header_style)
-                self.rowcounter += 1
 
             if self.merge_cells:
                 # Format hierarchical rows as merged cells.
@@ -1856,12 +1923,9 @@ class ExcelFormatter(object):
                                         header_style)
                     gcolidx += 1
 
-        # Get a frame that will account for any duplicates in the column names.
-        col_mapped_frame = self.df.loc[:, self.columns]
-
         # Write the body of the frame data series by series.
         for colidx in range(len(self.columns)):
-            series = col_mapped_frame.iloc[:, colidx]
+            series = self.df.iloc[:, colidx]
             for i, val in enumerate(series):
                 yield ExcelCell(self.rowcounter + i, gcolidx + colidx, val)
 
@@ -1877,10 +1941,17 @@ class ExcelFormatter(object):
 
 def format_array(values, formatter, float_format=None, na_rep='NaN',
                  digits=None, space=None, justify='right'):
-    if com.is_float_dtype(values.dtype):
+
+    if com.is_categorical_dtype(values):
+        fmt_klass = CategoricalArrayFormatter
+    elif com.is_float_dtype(values.dtype):
         fmt_klass = FloatArrayFormatter
+    elif com.is_period_arraylike(values):
+        fmt_klass = PeriodArrayFormatter
     elif com.is_integer_dtype(values.dtype):
         fmt_klass = IntArrayFormatter
+    elif com.is_datetimetz(values):
+        fmt_klass = Datetime64TZFormatter
     elif com.is_datetime64_dtype(values.dtype):
         fmt_klass = Datetime64Formatter
     elif com.is_timedelta64_dtype(values.dtype):
@@ -1937,6 +2008,8 @@ class GenericArrayFormatter(object):
             if self.na_rep is not None and lib.checknull(x):
                 if x is None:
                     return 'None'
+                elif x is pd.NaT:
+                    return 'NaT'
                 return self.na_rep
             elif isinstance(x, PandasObject):
                 return '%s' % x
@@ -1945,6 +2018,8 @@ class GenericArrayFormatter(object):
                 return '%s' % formatter(x)
 
         vals = self.values
+        if isinstance(vals, Index):
+            vals = vals._values
 
         is_float = lib.map_infer(vals, com.is_float) & notnull(vals)
         leading_space = is_float.any()
@@ -1996,7 +2071,7 @@ class FloatArrayFormatter(GenericArrayFormatter):
         if self.formatter is not None:
             fmt_values = [self.formatter(x) for x in self.values]
         else:
-            fmt_str = '%% .%df' % (self.digits - 1)
+            fmt_str = '%% .%df' % self.digits
             fmt_values = self._format_with(fmt_str)
 
             if len(fmt_values) > 0:
@@ -2004,20 +2079,20 @@ class FloatArrayFormatter(GenericArrayFormatter):
             else:
                 maxlen = 0
 
-            too_long = maxlen > self.digits + 5
+            too_long = maxlen > self.digits + 6
 
             abs_vals = np.abs(self.values)
 
             # this is pretty arbitrary for now
             has_large_values = (abs_vals > 1e8).any()
-            has_small_values = ((abs_vals < 10 ** (-self.digits+1)) &
+            has_small_values = ((abs_vals < 10 ** (-self.digits)) &
                                 (abs_vals > 0)).any()
 
             if too_long and has_large_values:
-                fmt_str = '%% .%de' % (self.digits - 1)
+                fmt_str = '%% .%de' % self.digits
                 fmt_values = self._format_with(fmt_str)
             elif has_small_values:
-                fmt_str = '%% .%de' % (self.digits - 1)
+                fmt_str = '%% .%de' % self.digits
                 fmt_values = self._format_with(fmt_str)
 
         return fmt_values
@@ -2027,9 +2102,7 @@ class IntArrayFormatter(GenericArrayFormatter):
 
     def _format_strings(self):
         formatter = self.formatter or (lambda x: '% d' % x)
-
         fmt_values = [formatter(x) for x in self.values]
-
         return fmt_values
 
 
@@ -2040,26 +2113,37 @@ class Datetime64Formatter(GenericArrayFormatter):
         self.date_format = date_format
 
     def _format_strings(self):
+        """ we by definition have DO NOT have a TZ """
 
-        # we may have a tz, if so, then need to process element-by-element
-        # when DatetimeBlockWithTimezones is a reality this could be fixed
         values = self.values
         if not isinstance(values, DatetimeIndex):
             values = DatetimeIndex(values)
 
-        if values.tz is None:
-            fmt_values = format_array_from_datetime(values.asi8.ravel(),
-                                                    format=_get_format_datetime64_from_values(values, self.date_format),
-                                                    na_rep=self.nat_rep).reshape(values.shape)
-            fmt_values = fmt_values.tolist()
+        fmt_values = format_array_from_datetime(values.asi8.ravel(),
+                                                format=_get_format_datetime64_from_values(values, self.date_format),
+                                                na_rep=self.nat_rep).reshape(values.shape)
+        return fmt_values.tolist()
 
-        else:
 
-            values = values.asobject
-            is_dates_only = _is_dates_only(values)
-            formatter = (self.formatter or _get_format_datetime64(is_dates_only, values, date_format=self.date_format))
-            fmt_values = [ formatter(x) for x in self.values ]
+class PeriodArrayFormatter(IntArrayFormatter):
 
+    def _format_strings(self):
+        values = PeriodIndex(self.values).to_native_types()
+        formatter = self.formatter or (lambda x: '%s' % x)
+        fmt_values = [formatter(x) for x in values]
+        return fmt_values
+
+
+class CategoricalArrayFormatter(GenericArrayFormatter):
+
+    def __init__(self, values, *args, **kwargs):
+        GenericArrayFormatter.__init__(self, values, *args, **kwargs)
+
+    def _format_strings(self):
+        fmt_values = format_array(self.values.get_values(), self.formatter,
+                                  float_format=self.float_format,
+                                  na_rep=self.na_rep, digits=self.digits,
+                                  space=self.space, justify=self.justify)
         return fmt_values
 
 
@@ -2114,9 +2198,21 @@ def _get_format_datetime64_from_values(values, date_format):
     is_dates_only = _is_dates_only(values)
     if is_dates_only:
         return date_format or "%Y-%m-%d"
-    return None
+    return date_format
 
 
+class Datetime64TZFormatter(Datetime64Formatter):
+
+    def _format_strings(self):
+        """ we by definition have a TZ """
+
+        values = self.values.asobject
+        is_dates_only = _is_dates_only(values)
+        formatter = (self.formatter or _get_format_datetime64(is_dates_only, date_format=self.date_format))
+        fmt_values = [ formatter(x) for x in values ]
+
+        return fmt_values
+
 class Timedelta64Formatter(GenericArrayFormatter):
 
     def __init__(self, values, nat_rep='NaT', box=False, **kwargs):
@@ -2127,7 +2223,7 @@ class Timedelta64Formatter(GenericArrayFormatter):
     def _format_strings(self):
         formatter = self.formatter or _get_format_timedelta64(self.values, nat_rep=self.nat_rep,
                                                               box=self.box)
-        fmt_values = [formatter(x) for x in self.values]
+        fmt_values = np.array([formatter(x) for x in self.values])
         return fmt_values
 
 
@@ -2168,13 +2264,16 @@ def _get_format_timedelta64(values, nat_rep='NaT', box=False):
     return _formatter
 
 
-def _make_fixed_width(strings, justify='right', minimum=None):
+def _make_fixed_width(strings, justify='right', minimum=None,
+                      adj=None):
+
     if len(strings) == 0 or justify == 'all':
         return strings
 
-    _strlen = _strlen_func()
+    if adj is None:
+        adj = _get_adjustment()
 
-    max_len = np.max([_strlen(x) for x in strings])
+    max_len = np.max([adj.len(x) for x in strings])
 
     if minimum is not None:
         max_len = max(minimum, max_len)
@@ -2183,22 +2282,14 @@ def _make_fixed_width(strings, justify='right', minimum=None):
     if conf_max is not None and max_len > conf_max:
         max_len = conf_max
 
-    if justify == 'left':
-        justfunc = lambda self, x: self.ljust(x)
-    else:
-        justfunc = lambda self, x: self.rjust(x)
-
     def just(x):
-        eff_len = max_len
-
         if conf_max is not None:
-            if (conf_max > 3) & (_strlen(x) > max_len):
-                x = x[:eff_len - 3] + '...'
-
-        return justfunc(x, eff_len)
-
-    result = [just(x) for x in strings]
+            if (conf_max > 3) & (adj.len(x) > max_len):
+                x = x[:max_len - 3] + '...'
+        return x
 
+    strings = [just(x) for x in strings]
+    result = adj.justify(strings, max_len, mode=justify)
     return result
 
 
