@@ -6,7 +6,7 @@ import pandas.compat as compat
 import pandas.core.common as com
 from pandas.core.common import (is_bool_indexer, is_integer_dtype,
                                 _asarray_tuplesafe, is_list_like, isnull,
-                                is_null_slice,
+                                is_null_slice, is_full_slice,
                                 ABCSeries, ABCDataFrame, ABCPanel, is_float,
                                 _values_from_object, _infer_fill_value, is_integer)
 import numpy as np
@@ -104,6 +104,8 @@ class _NDFrameIndexer(object):
 
         if isinstance(key, tuple) and not self.ndim < len(key):
             return self._convert_tuple(key, is_setter=True)
+        if isinstance(key, range):
+            return self._convert_range(key, is_setter=True)
 
         try:
             return self._convert_to_indexer(key, is_setter=True)
@@ -156,6 +158,10 @@ class _NDFrameIndexer(object):
                 keyidx.append(idx)
         return tuple(keyidx)
 
+    def _convert_range(self, key, is_setter=False):
+        """ convert a range argument """
+        return list(key)
+
     def _convert_scalar_indexer(self, key, axis):
         # if we are accessing via lowered dim, use the last dim
         ax = self.obj._get_axis(min(axis, self.ndim - 1))
@@ -201,6 +207,7 @@ class _NDFrameIndexer(object):
 
         # also has the side effect of consolidating in-place
         from pandas import Panel, DataFrame, Series
+        info_axis = self.obj._info_axis_number
 
         # maybe partial set
         take_split_path = self.obj._is_mixed_type
@@ -213,6 +220,16 @@ class _NDFrameIndexer(object):
                 val = list(value.values()) if isinstance(value,dict) else value
                 take_split_path = not blk._can_hold_element(val)
 
+        if isinstance(indexer, tuple) and len(indexer) == len(self.obj.axes):
+
+            for i, ax in zip(indexer, self.obj.axes):
+
+                # if we have any multi-indexes that have non-trivial slices (not null slices)
+                # then we must take the split path, xref GH 10360
+                if isinstance(ax, MultiIndex) and not (is_integer(i) or is_null_slice(i)):
+                    take_split_path = True
+                    break
+
         if isinstance(indexer, tuple):
             nindexer = []
             for i, idx in enumerate(indexer):
@@ -283,9 +300,9 @@ class _NDFrameIndexer(object):
                     new_index = index.insert(len(index),indexer)
 
                     # this preserves dtype of the value
-                    new_values = Series([value]).values
-                    if len(self.obj.values):
-                        new_values = np.concatenate([self.obj.values,
+                    new_values = Series([value])._values
+                    if len(self.obj._values):
+                        new_values = np.concatenate([self.obj._values,
                                                      new_values])
 
                     self.obj._data = self.obj._constructor(
@@ -328,14 +345,8 @@ class _NDFrameIndexer(object):
                     return self.obj.__setitem__(indexer, value)
 
         # set
-        info_axis = self.obj._info_axis_number
         item_labels = self.obj._get_axis(info_axis)
 
-        # if we have a complicated setup, take the split path
-        if (isinstance(indexer, tuple) and
-                any([isinstance(ax, MultiIndex) for ax in self.obj.axes])):
-            take_split_path = True
-
         # align and set the values
         if take_split_path:
 
@@ -399,10 +410,10 @@ class _NDFrameIndexer(object):
                 pi = plane_indexer[0] if lplane_indexer == 1 else plane_indexer
 
                 # perform the equivalent of a setitem on the info axis
-                # as we have a null slice which means essentially reassign to the columns
-                # of a multi-dim object
-                # GH6149
-                if isinstance(pi, tuple) and all(is_null_slice(idx) for idx in pi):
+                # as we have a null slice or a slice with full bounds
+                # which means essentially reassign to the columns of a multi-dim object
+                # GH6149 (null slice), GH10408 (full bounds)
+                if isinstance(pi, tuple) and all(is_null_slice(idx) or is_full_slice(idx, len(self.obj)) for idx in pi):
                     s = v
                 else:
                     # set the item, possibly having a dtype change
@@ -431,17 +442,24 @@ class _NDFrameIndexer(object):
 
                 return False
 
-            # we need an interable, with a ndim of at least 1
-            # eg. don't pass thru np.array(0)
+            # we need an iterable, with a ndim of at least 1
+            # eg. don't pass through np.array(0)
             if is_list_like_indexer(value) and getattr(value,'ndim',1) > 0:
 
                 # we have an equal len Frame
                 if isinstance(value, ABCDataFrame) and value.ndim > 1:
+                    sub_indexer = list(indexer)
+                    multiindex_indexer = isinstance(labels, MultiIndex)
 
                     for item in labels:
-                        # align to
-                        v = np.nan if item not in value else \
-                                self._align_series(indexer[0], value[item])
+                        if item in value:
+                            sub_indexer[info_axis] = item
+                            v = self._align_series(
+                                tuple(sub_indexer), value[item], multiindex_indexer
+                            )
+                        else:
+                            v = np.nan
+
                         setter(item, v)
 
                 # we have an equal len ndarray/convertible to our labels
@@ -507,9 +525,29 @@ class _NDFrameIndexer(object):
             self.obj._data = self.obj._data.setitem(indexer=indexer, value=value)
             self.obj._maybe_update_cacher(clear=True)
 
-    def _align_series(self, indexer, ser):
-        # indexer to assign Series can be tuple, slice, scalar
-        if isinstance(indexer, (slice, np.ndarray, list)):
+    def _align_series(self, indexer, ser, multiindex_indexer=False):
+        """
+        Parameters
+        ----------
+        indexer : tuple, slice, scalar
+            The indexer used to get the locations that will be set to
+            `ser`
+
+        ser : pd.Series
+            The values to assign to the locations specified by `indexer`
+
+        multiindex_indexer : boolean, optional
+            Defaults to False. Should be set to True if `indexer` was from
+            a `pd.MultiIndex`, to avoid unnecessary broadcasting.
+
+
+        Returns:
+        --------
+        `np.array` of `ser` broadcast to the appropriate shape for assignment
+        to the locations selected by `indexer`
+
+        """
+        if isinstance(indexer, (slice, np.ndarray, list, Index)):
             indexer = tuple([indexer])
 
         if isinstance(indexer, tuple):
@@ -543,10 +581,10 @@ class _NDFrameIndexer(object):
             # series, so need to broadcast (see GH5206)
             if (sum_aligners == self.ndim and
                     all([com.is_sequence(_) for _ in indexer])):
-                ser = ser.reindex(obj.axes[0][indexer[0]], copy=True).values
+                ser = ser.reindex(obj.axes[0][indexer[0]], copy=True)._values
 
                 # single indexer
-                if len(indexer) > 1:
+                if len(indexer) > 1 and not multiindex_indexer:
                     l = len(indexer[1])
                     ser = np.tile(ser, l).reshape(l, -1).T
 
@@ -565,9 +603,9 @@ class _NDFrameIndexer(object):
                     else:
                         new_ix = Index(new_ix)
                     if ser.index.equals(new_ix) or not len(new_ix):
-                        return ser.values.copy()
+                        return ser._values.copy()
 
-                    return ser.reindex(new_ix).values
+                    return ser.reindex(new_ix)._values
 
                 # 2 dims
                 elif single_aligner and is_frame:
@@ -575,8 +613,8 @@ class _NDFrameIndexer(object):
                     # reindex along index
                     ax = self.obj.axes[1]
                     if ser.index.equals(ax) or not len(ax):
-                        return ser.values.copy()
-                    return ser.reindex(ax).values
+                        return ser._values.copy()
+                    return ser.reindex(ax)._values
 
                 # >2 dims
                 elif single_aligner:
@@ -591,7 +629,7 @@ class _NDFrameIndexer(object):
                             broadcast.append((n, len(labels)))
 
                     # broadcast along other dims
-                    ser = ser.values.copy()
+                    ser = ser._values.copy()
                     for (axis, l) in broadcast:
                         shape = [-1] * (len(broadcast) + 1)
                         shape[axis] = l
@@ -606,9 +644,9 @@ class _NDFrameIndexer(object):
             ax = self.obj._get_axis(1)
 
             if ser.index.equals(ax):
-                return ser.values.copy()
+                return ser._values.copy()
 
-            return ser.reindex(ax).values
+            return ser.reindex(ax)._values
 
         raise ValueError('Incompatible indexer with Series')
 
@@ -654,16 +692,16 @@ class _NDFrameIndexer(object):
             if idx is not None and cols is not None:
 
                 if df.index.equals(idx) and df.columns.equals(cols):
-                    val = df.copy().values
+                    val = df.copy()._values
                 else:
-                    val = df.reindex(idx, columns=cols).values
+                    val = df.reindex(idx, columns=cols)._values
                 return val
 
         elif ((isinstance(indexer, slice) or is_list_like_indexer(indexer))
               and is_frame):
             ax = self.obj.index[indexer]
             if df.index.equals(ax):
-                val = df.copy().values
+                val = df.copy()._values
             else:
 
                 # we have a multi-index and are trying to align
@@ -672,7 +710,7 @@ class _NDFrameIndexer(object):
                     df.index, MultiIndex) and ax.nlevels != df.index.nlevels:
                     raise TypeError("cannot align on a multi-index with out specifying the join levels")
 
-                val = df.reindex(index=ax).values
+                val = df.reindex(index=ax)._values
             return val
 
         elif np.isscalar(indexer) and is_panel:
@@ -683,9 +721,9 @@ class _NDFrameIndexer(object):
             # a passed in dataframe which is actually a transpose
             # of what is needed
             if idx.equals(df.index) and cols.equals(df.columns):
-                return df.copy().values
+                return df.copy()._values
 
-            return df.reindex(idx, columns=cols).values
+            return df.reindex(idx, columns=cols)._values
 
         raise ValueError('Incompatible indexer with DataFrame')
 
@@ -1031,7 +1069,7 @@ class _NDFrameIndexer(object):
         # if we are a label return me
         try:
             return labels.get_loc(obj)
-        except KeyError:
+        except LookupError:
             if isinstance(obj, tuple) and isinstance(labels, MultiIndex):
                 if is_setter and len(obj) == labels.nlevels:
                     return {'key': obj}
@@ -1116,7 +1154,7 @@ class _NDFrameIndexer(object):
         else:
             try:
                 return labels.get_loc(obj)
-            except KeyError:
+            except LookupError:
                 # allow a not found key only if we are a setter
                 if not is_list_like_indexer(obj) and is_setter:
                     return {'key': obj}
@@ -1276,7 +1314,7 @@ class _LocIndexer(_LocationIndexer):
 
             def error():
                 if isnull(key):
-                    raise ValueError(
+                    raise TypeError(
                         "cannot use label indexing with a null key")
                 raise KeyError("the label [%s] is not in the [%s]" %
                                (key, self.obj._get_axis_name(axis)))
@@ -1388,7 +1426,8 @@ class _iLocIndexer(_LocationIndexer):
         # return a boolean if we have a valid integer indexer
 
         ax = self.obj._get_axis(axis)
-        if key > len(ax):
+        l = len(ax)
+        if key >= l or key < -l:
             raise IndexError("single positional indexer is out-of-bounds")
         return True
 
@@ -1400,7 +1439,7 @@ class _iLocIndexer(_LocationIndexer):
         arr = np.array(key)
         ax = self.obj._get_axis(axis)
         l = len(ax)
-        if len(arr) and (arr.max() >= l or arr.min() <= -l):
+        if len(arr) and (arr.max() >= l or arr.min() < -l):
             raise IndexError("positional indexers are out-of-bounds")
 
         return True
@@ -1654,11 +1693,11 @@ def check_bool_indexer(ax, key):
     result = key
     if isinstance(key, ABCSeries) and not key.index.equals(ax):
         result = result.reindex(ax)
-        mask = com.isnull(result.values)
+        mask = com.isnull(result._values)
         if mask.any():
             raise IndexingError('Unalignable boolean Series key provided')
 
-        result = result.astype(bool).values
+        result = result.astype(bool)._values
 
     else:
         # is_bool_indexer has already checked for nulls in the case of an
@@ -1719,7 +1758,7 @@ def maybe_convert_ix(*args):
 
     ixify = True
     for arg in args:
-        if not isinstance(arg, (np.ndarray, list, ABCSeries)):
+        if not isinstance(arg, (np.ndarray, list, ABCSeries, Index)):
             ixify = False
 
     if ixify:
@@ -1773,3 +1812,46 @@ def maybe_droplevels(index, key):
             pass
 
     return index
+
+
+def _non_reducing_slice(slice_):
+    """
+    Ensurse that a slice doesn't reduce to a Series or Scalar.
+
+    Any user-paseed `subset` should have this called on it
+    to make sure we're always working with DataFrames.
+    """
+    # default to column slice, like DataFrame
+    # ['A', 'B'] -> IndexSlices[:, ['A', 'B']]
+    kinds = tuple(list(compat.string_types) +
+                  [ABCSeries, np.ndarray, Index, list])
+    if isinstance(slice_, kinds):
+        slice_ = IndexSlice[:, slice_]
+
+    def pred(part):
+        # true when slice does *not* reduce
+        return isinstance(part, slice) or com.is_list_like(part)
+
+    if not com.is_list_like(slice_):
+        if not isinstance(slice_, slice):
+            # a 1-d slice, like df.loc[1]
+            slice_ = [[slice_]]
+        else:
+            # slice(a, b, c)
+            slice_ = [slice_]  # to tuplize later
+    else:
+        slice_ = [part if pred(part) else [part] for part in slice_]
+    return tuple(slice_)
+
+
+def _maybe_numeric_slice(df, slice_, include_bool=False):
+    """
+    want nice defaults for background_gradient that don't break
+    with non-numeric data. But if slice_ is passed go with that.
+    """
+    if slice_ is None:
+        dtypes = [np.number]
+        if include_bool:
+            dtypes.append(bool)
+        slice_ = IndexSlice[:, df.select_dtypes(include=dtypes).columns]
+    return slice_
