@@ -1,5 +1,5 @@
 """ io on the clipboard """
-from pandas import compat, get_option, DataFrame
+from pandas import compat, get_option, option_context, DataFrame
 from pandas.compat import StringIO
 
 
@@ -14,13 +14,12 @@ def read_clipboard(**kwargs):  # pragma: no cover
     -------
     parsed : DataFrame
     """
-    if kwargs.get('sep') is None and kwargs.get('delim_whitespace') is None:
-        kwargs['sep'] = '\s+'
     from pandas.util.clipboard import clipboard_get
     from pandas.io.parsers import read_table
     text = clipboard_get()
 
     # try to decode (if needed on PY3)
+    # Strange. linux py33 doesn't complain, win py33 does
     if compat.PY3:
         try:
             text = compat.bytes_to_str(
@@ -29,6 +28,26 @@ def read_clipboard(**kwargs):  # pragma: no cover
             )
         except:
             pass
+
+    # Excel copies into clipboard with \t seperation
+    # inspect no more then the 10 first lines, if they
+    # all contain an equal number (>0) of tabs, infer
+    # that this came from excel and set 'sep' accordingly
+    lines = text[:10000].split('\n')[:-1][:10]
+
+    # Need to remove leading white space, since read_table
+    # accepts:
+    #    a  b
+    # 0  1  2
+    # 1  3  4
+
+    counts = set([x.lstrip().count('\t') for x in lines])
+    if len(lines)>1 and len(counts) == 1 and counts.pop() != 0:
+        kwargs['sep'] = '\t'
+
+    if kwargs.get('sep') is None and kwargs.get('delim_whitespace') is None:
+        kwargs['sep'] = '\s+'
+
     return read_table(StringIO(text), **kwargs)
 
 
@@ -72,7 +91,8 @@ def to_clipboard(obj, excel=None, sep=None, **kwargs):  # pragma: no cover
 
     if isinstance(obj, DataFrame):
         # str(df) has various unhelpful defaults, like truncation
-        objstr = obj.to_string()
+        with option_context('display.max_colwidth', 999999):
+            objstr = obj.to_string(**kwargs)
     else:
         objstr = str(obj)
     clipboard_set(objstr)
