@@ -20,7 +20,7 @@ from numpy.testing.decorators import slow
 
 from pandas import (DataFrame, MultiIndex, read_csv, Timestamp, Index,
                     date_range, Series)
-from pandas.compat import map, zip, StringIO, string_types, BytesIO
+from pandas.compat import map, zip, StringIO, string_types, BytesIO, is_platform_windows
 from pandas.io.common import URLError, urlopen, file_path_to_url
 from pandas.io.html import read_html
 from pandas.parser import CParserError
@@ -86,27 +86,22 @@ def test_bs4_version_fails():
                          flavor='bs4')
 
 
-class TestReadHtml(tm.TestCase):
+class ReadHtmlMixin(object):
+    def read_html(self, *args, **kwargs):
+        kwargs.setdefault('flavor', self.flavor)
+        return read_html(*args, **kwargs)
+
+
+class TestReadHtml(tm.TestCase, ReadHtmlMixin):
+    flavor = 'bs4'
+    spam_data = os.path.join(DATA_PATH, 'spam.html')
+    banklist_data = os.path.join(DATA_PATH, 'banklist.html')
+
     @classmethod
     def setUpClass(cls):
         super(TestReadHtml, cls).setUpClass()
         _skip_if_none_of(('bs4', 'html5lib'))
 
-    def read_html(self, *args, **kwargs):
-        kwargs['flavor'] = kwargs.get('flavor', self.flavor)
-        return read_html(*args, **kwargs)
-
-    def setup_data(self):
-        self.spam_data = os.path.join(DATA_PATH, 'spam.html')
-        self.banklist_data = os.path.join(DATA_PATH, 'banklist.html')
-
-    def setup_flavor(self):
-        self.flavor = 'bs4'
-
-    def setUp(self):
-        self.setup_data()
-        self.setup_flavor()
-
     def test_to_html_compat(self):
         df = mkdf(4, 3, data_gen_f=lambda *args: rand(), c_idx_names=False,
                   r_idx_names=False).applymap('{0:.3f}'.format).astype(float)
@@ -142,12 +137,10 @@ class TestReadHtml(tm.TestCase):
         assert_framelist_equal(df1, df2)
 
     def test_spam_no_types(self):
-        with tm.assert_produces_warning(FutureWarning):
-            df1 = self.read_html(self.spam_data, '.*Water.*',
-                                 infer_types=False)
-        with tm.assert_produces_warning(FutureWarning):
-            df2 = self.read_html(self.spam_data, 'Unit', infer_types=False)
 
+        # infer_types removed in #10892
+        df1 = self.read_html(self.spam_data, '.*Water.*')
+        df2 = self.read_html(self.spam_data, 'Unit')
         assert_framelist_equal(df1, df2)
 
         self.assertEqual(df1[0].ix[0, 0], 'Proximates')
@@ -164,12 +157,12 @@ class TestReadHtml(tm.TestCase):
     def test_spam_no_match(self):
         dfs = self.read_html(self.spam_data)
         for df in dfs:
-            tm.assert_isinstance(df, DataFrame)
+            tm.assertIsInstance(df, DataFrame)
 
     def test_banklist_no_match(self):
         dfs = self.read_html(self.banklist_data, attrs={'id': 'table'})
         for df in dfs:
-            tm.assert_isinstance(df, DataFrame)
+            tm.assertIsInstance(df, DataFrame)
 
     def test_spam_header(self):
         df = self.read_html(self.spam_data, '.*Water.*', header=1)[0]
@@ -235,12 +228,9 @@ class TestReadHtml(tm.TestCase):
         assert_framelist_equal(df1, df2)
 
     def test_header_and_index_no_types(self):
-        with tm.assert_produces_warning(FutureWarning):
-            df1 = self.read_html(self.spam_data, '.*Water.*', header=1,
-                                 index_col=0, infer_types=False)
-        with tm.assert_produces_warning(FutureWarning):
-            df2 = self.read_html(self.spam_data, 'Unit', header=1, index_col=0,
-                                 infer_types=False)
+        df1 = self.read_html(self.spam_data, '.*Water.*', header=1,
+                             index_col=0)
+        df2 = self.read_html(self.spam_data, 'Unit', header=1, index_col=0)
         assert_framelist_equal(df1, df2)
 
     def test_header_and_index_with_types(self):
@@ -250,21 +240,12 @@ class TestReadHtml(tm.TestCase):
         assert_framelist_equal(df1, df2)
 
     def test_infer_types(self):
-        with tm.assert_produces_warning(FutureWarning):
-            df1 = self.read_html(self.spam_data, '.*Water.*', index_col=0,
-                                 infer_types=False)
-        with tm.assert_produces_warning(FutureWarning):
-            df2 = self.read_html(self.spam_data, 'Unit', index_col=0,
-                                 infer_types=False)
+
+        # 10892 infer_types removed
+        df1 = self.read_html(self.spam_data, '.*Water.*', index_col=0)
+        df2 = self.read_html(self.spam_data, 'Unit', index_col=0)
         assert_framelist_equal(df1, df2)
 
-        with tm.assert_produces_warning(FutureWarning):
-            df2 = self.read_html(self.spam_data, 'Unit', index_col=0,
-                                 infer_types=True)
-
-        with tm.assertRaises(AssertionError):
-            assert_framelist_equal(df1, df2)
-
     def test_string_io(self):
         with open(self.spam_data) as f:
             data1 = StringIO(f.read())
@@ -313,9 +294,9 @@ class TestReadHtml(tm.TestCase):
         url = self.banklist_data
         dfs = self.read_html(file_path_to_url(url), 'First',
                              attrs={'id': 'table'})
-        tm.assert_isinstance(dfs, list)
+        tm.assertIsInstance(dfs, list)
         for df in dfs:
-            tm.assert_isinstance(df, DataFrame)
+            tm.assertIsInstance(df, DataFrame)
 
     @slow
     def test_invalid_table_attrs(self):
@@ -331,34 +312,34 @@ class TestReadHtml(tm.TestCase):
     @slow
     def test_multiindex_header(self):
         df = self._bank_data(header=[0, 1])[0]
-        tm.assert_isinstance(df.columns, MultiIndex)
+        tm.assertIsInstance(df.columns, MultiIndex)
 
     @slow
     def test_multiindex_index(self):
         df = self._bank_data(index_col=[0, 1])[0]
-        tm.assert_isinstance(df.index, MultiIndex)
+        tm.assertIsInstance(df.index, MultiIndex)
 
     @slow
     def test_multiindex_header_index(self):
         df = self._bank_data(header=[0, 1], index_col=[0, 1])[0]
-        tm.assert_isinstance(df.columns, MultiIndex)
-        tm.assert_isinstance(df.index, MultiIndex)
+        tm.assertIsInstance(df.columns, MultiIndex)
+        tm.assertIsInstance(df.index, MultiIndex)
 
     @slow
     def test_multiindex_header_skiprows_tuples(self):
         df = self._bank_data(header=[0, 1], skiprows=1, tupleize_cols=True)[0]
-        tm.assert_isinstance(df.columns, Index)
+        tm.assertIsInstance(df.columns, Index)
 
     @slow
     def test_multiindex_header_skiprows(self):
         df = self._bank_data(header=[0, 1], skiprows=1)[0]
-        tm.assert_isinstance(df.columns, MultiIndex)
+        tm.assertIsInstance(df.columns, MultiIndex)
 
     @slow
     def test_multiindex_header_index_skiprows(self):
         df = self._bank_data(header=[0, 1], index_col=[0, 1], skiprows=1)[0]
-        tm.assert_isinstance(df.index, MultiIndex)
-        tm.assert_isinstance(df.columns, MultiIndex)
+        tm.assertIsInstance(df.index, MultiIndex)
+        tm.assertIsInstance(df.columns, MultiIndex)
 
     @slow
     def test_regex_idempotency(self):
@@ -366,9 +347,9 @@ class TestReadHtml(tm.TestCase):
         dfs = self.read_html(file_path_to_url(url),
                              match=re.compile(re.compile('Florida')),
                              attrs={'id': 'table'})
-        tm.assert_isinstance(dfs, list)
+        tm.assertIsInstance(dfs, list)
         for df in dfs:
-            tm.assert_isinstance(df, DataFrame)
+            tm.assertIsInstance(df, DataFrame)
 
     def test_negative_skiprows(self):
         with tm.assertRaisesRegexp(ValueError,
@@ -377,16 +358,16 @@ class TestReadHtml(tm.TestCase):
 
     @network
     def test_multiple_matches(self):
-        url = 'http://code.google.com/p/pythonxy/wiki/StandardPlugins'
-        dfs = self.read_html(url, match='Python', attrs={'class': 'wikitable'})
+        url = 'https://docs.python.org/2/'
+        dfs = self.read_html(url, match='Python')
         self.assertTrue(len(dfs) > 1)
 
     @network
-    def test_pythonxy_plugins_table(self):
-        url = 'http://code.google.com/p/pythonxy/wiki/StandardPlugins'
-        dfs = self.read_html(url, match='Python', attrs={'class': 'wikitable'})
-        zz = [df.iloc[0, 0] for df in dfs]
-        self.assertEqual(sorted(zz), sorted(['Python', 'SciTE']))
+    def test_python_docs_table(self):
+        url = 'https://docs.python.org/2/'
+        dfs = self.read_html(url, match='Python')
+        zz = [df.iloc[0, 0][0:4] for df in dfs]
+        self.assertEqual(sorted(zz), sorted(['Repo', 'What']))
 
     @slow
     def test_thousands_macau_stats(self):
@@ -407,6 +388,64 @@ class TestReadHtml(tm.TestCase):
 
         self.assertFalse(any(s.isnull().any() for _, s in df.iteritems()))
 
+    def test_empty_tables(self):
+        """
+        Make sure that read_html ignores empty tables.
+        """
+        data1 = '''<table>
+            <thead>
+                <tr>
+                    <th>A</th>
+                    <th>B</th>
+                </tr>
+            </thead>
+            <tbody>
+                <tr>
+                    <td>1</td>
+                    <td>2</td>
+                </tr>
+            </tbody>
+        </table>'''
+        data2 = data1 + '''<table>
+            <tbody>
+            </tbody>
+        </table>'''
+        res1 = self.read_html(StringIO(data1))
+        res2 = self.read_html(StringIO(data2))
+        assert_framelist_equal(res1, res2)
+
+    def test_tfoot_read(self):
+        """
+        Make sure that read_html reads tfoot, containing td or th.
+        Ignores empty tfoot
+        """
+        data_template = '''<table>
+            <thead>
+                <tr>
+                    <th>A</th>
+                    <th>B</th>
+                </tr>
+            </thead>
+            <tbody>
+                <tr>
+                    <td>bodyA</td>
+                    <td>bodyB</td>
+                </tr>
+            </tbody>
+            <tfoot>
+                {footer}
+            </tfoot>
+        </table>'''
+
+        data1 = data_template.format(footer = "")
+        data2 = data_template.format(footer ="<tr><td>footA</td><th>footB</th></tr>")
+
+        d1 = {'A': ['bodyA'], 'B': ['bodyB']}
+        d2 = {'A': ['bodyA', 'footA'], 'B': ['bodyB', 'footB']}
+
+        tm.assert_frame_equal(self.read_html(data1)[0], DataFrame(d1))
+        tm.assert_frame_equal(self.read_html(data2)[0], DataFrame(d2))
+
     def test_countries_municipalities(self):
         # GH5048
         data1 = StringIO('''<table>
@@ -488,9 +527,11 @@ class TestReadHtml(tm.TestCase):
                'Hamilton Bank, NA', 'The Citizens Savings Bank']
         dfnew = df.applymap(try_remove_ws).replace(old, new)
         gtnew = ground_truth.applymap(try_remove_ws)
-        converted = dfnew.convert_objects(convert_numeric=True)
-        tm.assert_frame_equal(converted.convert_objects(convert_dates='coerce'),
-                              gtnew)
+        converted = dfnew._convert(datetime=True, numeric=True)
+        date_cols = ['Closing Date','Updated Date']
+        converted[date_cols] = converted[date_cols]._convert(datetime=True,
+                                                             coerce=True)
+        tm.assert_frame_equal(converted,gtnew)
 
     @slow
     def test_gold_canyon(self):
@@ -568,7 +609,9 @@ class TestReadHtml(tm.TestCase):
     def test_parse_dates_list(self):
         df = DataFrame({'date': date_range('1/1/2001', periods=10)})
         expected = df.to_html()
-        res = self.read_html(expected, parse_dates=[0], index_col=0)
+        res = self.read_html(expected, parse_dates=[1], index_col=0)
+        tm.assert_frame_equal(df, res[0])
+        res = self.read_html(expected, parse_dates=['date'], index_col=0)
         tm.assert_frame_equal(df, res[0])
 
     def test_parse_dates_combine(self):
@@ -585,9 +628,20 @@ class TestReadHtml(tm.TestCase):
         with tm.assertRaisesRegexp(CParserError, r"Passed header=\[0,1\] are "
                                    "too many rows for this multi_index "
                                    "of columns"):
-            with tm.assert_produces_warning(FutureWarning):
-                self.read_html(data, infer_types=False, header=[0, 1])
+            self.read_html(data, header=[0, 1])
 
+    def test_wikipedia_states_table(self):
+        data = os.path.join(DATA_PATH, 'wikipedia_states.html')
+        assert os.path.isfile(data), '%r is not a file' % data
+        assert os.path.getsize(data), '%r is an empty file' % data
+        result = self.read_html(data, 'Arizona', header=1)[0]
+        nose.tools.assert_equal(result['sq mi'].dtype, np.dtype('float64'))
+
+    def test_bool_header_arg(self):
+        #GH 6114
+        for arg in [True, False]:
+            with tm.assertRaises(TypeError):
+                read_html(self.spam_data, header=arg)
 
 def _lang_enc(filename):
     return os.path.splitext(os.path.basename(filename))[0].split('_')
@@ -595,56 +649,80 @@ def _lang_enc(filename):
 
 class TestReadHtmlEncoding(tm.TestCase):
     files = glob.glob(os.path.join(DATA_PATH, 'html_encoding', '*.html'))
+    flavor = 'bs4'
+
+    @classmethod
+    def setUpClass(cls):
+        super(TestReadHtmlEncoding, cls).setUpClass()
+        _skip_if_none_of((cls.flavor, 'html5lib'))
+
+    def read_html(self, *args, **kwargs):
+        kwargs['flavor'] = self.flavor
+        return read_html(*args, **kwargs)
 
     def read_filename(self, f, encoding):
-        return read_html(f, encoding=encoding, index_col=0)
+        return self.read_html(f, encoding=encoding, index_col=0)
 
     def read_file_like(self, f, encoding):
         with open(f, 'rb') as fobj:
-            return read_html(BytesIO(fobj.read()), encoding=encoding,
-                             index_col=0)
+            return self.read_html(BytesIO(fobj.read()), encoding=encoding,
+                                  index_col=0)
 
     def read_string(self, f, encoding):
         with open(f, 'rb') as fobj:
-            return read_html(fobj.read(), encoding=encoding, index_col=0)
+            return self.read_html(fobj.read(), encoding=encoding, index_col=0)
 
     def test_encode(self):
+        assert self.files, 'no files read from the data folder'
         for f in self.files:
             _, encoding = _lang_enc(f)
-            from_string = self.read_string(f, encoding).pop()
-            from_file_like = self.read_file_like(f, encoding).pop()
-            from_filename = self.read_filename(f, encoding).pop()
-            tm.assert_frame_equal(from_string, from_file_like)
-            tm.assert_frame_equal(from_string, from_filename)
+            try:
+                from_string = self.read_string(f, encoding).pop()
+                from_file_like = self.read_file_like(f, encoding).pop()
+                from_filename = self.read_filename(f, encoding).pop()
+                tm.assert_frame_equal(from_string, from_file_like)
+                tm.assert_frame_equal(from_string, from_filename)
+            except Exception as e:
+
+                # seems utf-16/32 fail on windows
+                if is_platform_windows():
+                    if '16' in encoding or '32' in encoding:
+                        continue
+                    raise
+
+class TestReadHtmlEncodingLxml(TestReadHtmlEncoding):
+    flavor = 'lxml'
+
+    @classmethod
+    def setUpClass(cls):
+        super(TestReadHtmlEncodingLxml, cls).setUpClass()
+        _skip_if_no(cls.flavor)
 
 
-class TestReadHtmlLxml(tm.TestCase):
+class TestReadHtmlLxml(tm.TestCase, ReadHtmlMixin):
+    flavor = 'lxml'
+
     @classmethod
     def setUpClass(cls):
         super(TestReadHtmlLxml, cls).setUpClass()
         _skip_if_no('lxml')
 
-    def read_html(self, *args, **kwargs):
-        self.flavor = ['lxml']
-        kwargs['flavor'] = kwargs.get('flavor', self.flavor)
-        return read_html(*args, **kwargs)
-
     def test_data_fail(self):
         from lxml.etree import XMLSyntaxError
         spam_data = os.path.join(DATA_PATH, 'spam.html')
         banklist_data = os.path.join(DATA_PATH, 'banklist.html')
 
         with tm.assertRaises(XMLSyntaxError):
-            self.read_html(spam_data, flavor=['lxml'])
+            self.read_html(spam_data)
 
         with tm.assertRaises(XMLSyntaxError):
-            self.read_html(banklist_data, flavor=['lxml'])
+            self.read_html(banklist_data)
 
     def test_works_on_valid_markup(self):
         filename = os.path.join(DATA_PATH, 'valid_markup.html')
-        dfs = self.read_html(filename, index_col=0, flavor=['lxml'])
-        tm.assert_isinstance(dfs, list)
-        tm.assert_isinstance(dfs[0], DataFrame)
+        dfs = self.read_html(filename, index_col=0)
+        tm.assertIsInstance(dfs, list)
+        tm.assertIsInstance(dfs[0], DataFrame)
 
     @slow
     def test_fallback_success(self):
@@ -655,7 +733,9 @@ class TestReadHtmlLxml(tm.TestCase):
     def test_parse_dates_list(self):
         df = DataFrame({'date': date_range('1/1/2001', periods=10)})
         expected = df.to_html()
-        res = self.read_html(expected, parse_dates=[0], index_col=0)
+        res = self.read_html(expected, parse_dates=[1], index_col=0)
+        tm.assert_frame_equal(df, res[0])
+        res = self.read_html(expected, parse_dates=['date'], index_col=0)
         tm.assert_frame_equal(df, res[0])
 
     def test_parse_dates_combine(self):
@@ -669,14 +749,13 @@ class TestReadHtmlLxml(tm.TestCase):
 
     def test_computer_sales_page(self):
         data = os.path.join(DATA_PATH, 'computer_sales_page.html')
-        with tm.assert_produces_warning(FutureWarning):
-            self.read_html(data, infer_types=False, header=[0, 1])
+        self.read_html(data, header=[0, 1])
 
 
 def test_invalid_flavor():
     url = 'google.com'
-    nose.tools.assert_raises(ValueError, read_html, url, 'google',
-                             flavor='not a* valid**++ flaver')
+    with tm.assertRaises(ValueError):
+        read_html(url, 'google', flavor='not a* valid**++ flaver')
 
 
 def get_elements_from_file(url, element='table'):
