@@ -117,6 +117,7 @@ fail:
     return -1;
 }
 
+#if 0
 /*
  * Wraps `gmtime` functionality for multiple platforms. This
  * converts a time value to a time structure in UTC.
@@ -161,6 +162,7 @@ fail:
                                 "to a UTC time", func_name);
     return -1;
 }
+#endif
 
 /*
  * Converts a datetimestruct in UTC to a datetimestruct in local time,
@@ -363,7 +365,10 @@ convert_datetimestruct_local_to_utc(pandas_datetimestruct *out_dts_utc,
  *           to be cast to the 'unit' parameter.
  *
  * 'out' gets filled with the parsed date-time.
- * 'out_local' gets set to 1 if the parsed time was in local time,
+ * 'out_local' gets set to 1 if the parsed time contains timezone, 
+ *      to 0 otherwise.
+ * 'out_tzoffset' gets set to timezone offset by minutes
+ *      if the parsed time was in local time,
  *      to 0 otherwise. The values 'now' and 'today' don't get counted
  *      as local, and neither do UTC +/-#### timezone offsets, because
  *      they aren't using the computer's local timezone offset.
@@ -381,7 +386,8 @@ parse_iso_8601_datetime(char *str, int len,
                     PANDAS_DATETIMEUNIT unit,
                     NPY_CASTING casting,
                     pandas_datetimestruct *out,
-                    npy_bool *out_local,
+                    int *out_local,
+                    int *out_tzoffset,
                     PANDAS_DATETIMEUNIT *out_bestunit,
                     npy_bool *out_special)
 {
@@ -778,28 +784,19 @@ parse_timezone:
     if (sublen == 0) {
         // Unlike NumPy, treating no time zone as naive
         goto finish;
-
-/*
-        if (convert_datetimestruct_local_to_utc(out, out) < 0) {
-            goto error;
-        }
-
-        // Since neither "Z" nor a time-zone was specified, it's local
-        if (out_local != NULL) {
-            *out_local = 1;
-        }
-
-        goto finish;
-*/
     }
 
     /* UTC specifier */
     if (*substr == 'Z') {
-        /* "Z" means not local */
+        /* "Z" should be equivalent to tz offset "+00:00" */
         if (out_local != NULL) {
-            *out_local = 0;
+            *out_local = 1;
         }
 
+        if (out_tzoffset != NULL) {
+            *out_tzoffset = 0;
+         }
+
         if (sublen == 1) {
             goto finish;
         }
@@ -816,9 +813,6 @@ parse_timezone:
          * Since "local" means local with respect to the current
          * machine, we say this is non-local.
          */
-        if (out_local != NULL) {
-            *out_local = 0;
-        }
 
         if (*substr == '-') {
             offset_neg = 1;
@@ -872,7 +866,11 @@ parse_timezone:
             offset_hour = -offset_hour;
             offset_minute = -offset_minute;
         }
-        add_minutes_to_datetimestruct(out, -60 * offset_hour - offset_minute);
+        if (out_local != NULL) {
+            *out_local = 1;
+            // Unlike NumPy, do not change internal value to local time
+            *out_tzoffset = 60 * offset_hour - offset_minute;
+        }
     }
 
     /* Skip trailing whitespace */
@@ -1148,7 +1146,7 @@ make_iso_8601_datetime(pandas_datetimestruct *dts, char *outstr, int outlen,
 #ifdef _WIN32
     tmplen = _snprintf(substr, sublen, "%04" NPY_INT64_FMT, dts->year);
 #else
-    tmplen = snprintf(substr, sublen, "%04" NPY_INT64_FMT, (long long)dts->year);
+    tmplen = snprintf(substr, sublen, "%04" NPY_INT64_FMT, dts->year);
 #endif
     /* If it ran out of space or there isn't space for the NULL terminator */
     if (tmplen < 0 || tmplen > sublen) {
