@@ -80,8 +80,8 @@ span : float, optional
 halflife : float, optional
     Specify decay in terms of halflife, :math:`\alpha = 1 - exp(log(0.5) / halflife)`
 min_periods : int, default 0
-    Number of observations in sample to require (only affects
-    beginning)
+    Minimum number of observations in window required to have a value
+    (otherwise result is NA).
 freq : None or string alias / date offset object, default=None
     Frequency to conform to before computing statistic
 adjust : boolean, default True
@@ -89,12 +89,15 @@ adjust : boolean, default True
     imbalance in relative weightings (viewing EWMA as a moving average)
 how : string, default 'mean'
     Method for down- or re-sampling
+ignore_na : boolean, default False
+    Ignore missing values when calculating weights;
+    specify True to reproduce pre-0.15.0 behavior
 """
 
 _ewm_notes = r"""
 Notes
 -----
-Either center of mass or span must be specified
+Either center of mass, span or halflife must be specified
 
 EWMA is sometimes specified using a "span" parameter `s`, we have that the
 decay parameter :math:`\alpha` is related to the span as
@@ -104,6 +107,26 @@ where `c` is the center of mass. Given a span, the associated center of mass is
 :math:`c = (s - 1) / 2`
 
 So a "20-day EWMA" would have center 9.5.
+
+When adjust is True (default), weighted averages are calculated using weights
+    (1-alpha)**(n-1), (1-alpha)**(n-2), ..., 1-alpha, 1.
+
+When adjust is False, weighted averages are calculated recursively as:
+    weighted_average[0] = arg[0];
+    weighted_average[i] = (1-alpha)*weighted_average[i-1] + alpha*arg[i].
+
+When ignore_na is False (default), weights are based on absolute positions.
+For example, the weights of x and y used in calculating the final weighted
+average of [x, None, y] are (1-alpha)**2 and 1 (if adjust is True), and
+(1-alpha)**2 and alpha (if adjust is False).
+
+When ignore_na is True (reproducing pre-0.15.0 behavior), weights are based on
+relative positions. For example, the weights of x and y used in calculating
+the final weighted average of [x, None, y] are 1-alpha and 1 (if adjust is
+True), and 1-alpha and alpha (if adjust is False).
+
+More details can be found at
+http://pandas.pydata.org/pandas-docs/stable/computation.html#exponentially-weighted-moment-functions
 """
 
 _expanding_kw = """min_periods : int, default None
@@ -147,6 +170,11 @@ _pairwise_kw = """pairwise : bool, default False
     elements, only complete pairwise observations will be used.
 """
 
+_ddof_kw = """ddof : int, default 1
+    Delta Degrees of Freedom.  The divisor used in calculations
+    is ``N - ddof``, where ``N`` represents the number of elements.
+"""
+
 _bias_kw = r"""bias : boolean, default False
     Use a standard estimation bias correction
 """
@@ -181,25 +209,25 @@ def rolling_count(arg, window, freq=None, center=False, how=None):
     of :meth:`~pandas.Series.resample` (i.e. using the `mean`).
     """
     arg = _conv_timerule(arg, freq, how)
-    window = min(window, len(arg))
+    if not center:
+        window = min(window, len(arg))
 
     return_hook, values = _process_data_structure(arg, kill_inf=False)
 
     converted = np.isfinite(values).astype(float)
-    result = rolling_sum(converted, window, min_periods=1,
+    result = rolling_sum(converted, window, min_periods=0,
                          center=center)  # already converted
 
     # putmask here?
     result[np.isnan(result)] = 0
-
     return return_hook(result)
 
 
 @Substitution("Unbiased moving covariance.", _binary_arg_flex,
-              _roll_kw%'None'+_pairwise_kw, _flex_retval, _roll_notes)
+              _roll_kw%'None'+_pairwise_kw+_ddof_kw, _flex_retval, _roll_notes)
 @Appender(_doc_template)
 def rolling_cov(arg1, arg2=None, window=None, min_periods=None, freq=None,
-                center=False, pairwise=None, how=None):
+                center=False, pairwise=None, how=None, ddof=1):
     if window is None and isinstance(arg2, (int, float)):
         window = arg2
         arg2 = arg1
@@ -211,10 +239,9 @@ def rolling_cov(arg1, arg2=None, window=None, min_periods=None, freq=None,
     arg2 = _conv_timerule(arg2, freq, how)
 
     def _get_cov(X, Y):
-        adj_window = min(window, len(X), len(Y))
-        mean = lambda x: rolling_mean(x, adj_window, min_periods, center=center)
-        count = rolling_count(X + Y, adj_window, center=center)
-        bias_adj = count / (count - 1)
+        mean = lambda x: rolling_mean(x, window, min_periods, center=center)
+        count = rolling_count(X + Y, window, center=center)
+        bias_adj = count / (count - ddof)
         return (mean(X * Y) - mean(X) * mean(Y)) * bias_adj
     rs = _flex_binary_moment(arg1, arg2, _get_cov, pairwise=bool(pairwise))
     return rs
@@ -236,12 +263,11 @@ def rolling_corr(arg1, arg2=None, window=None, min_periods=None, freq=None,
     arg2 = _conv_timerule(arg2, freq, how)
 
     def _get_corr(a, b):
-        adj_window = min(window, len(a), len(b))
-        num = rolling_cov(a, b, adj_window, min_periods, freq=freq,
+        num = rolling_cov(a, b, window, min_periods, freq=freq,
                           center=center)
-        den = (rolling_std(a, adj_window, min_periods, freq=freq,
+        den = (rolling_std(a, window, min_periods, freq=freq,
                            center=center) *
-               rolling_std(b, adj_window, min_periods, freq=freq,
+               rolling_std(b, window, min_periods, freq=freq,
                            center=center))
         return num / den
 
@@ -258,38 +284,58 @@ def _flex_binary_moment(arg1, arg2, f, pairwise=False):
             isinstance(arg2, (np.ndarray,Series)):
         X, Y = _prep_binary(arg1, arg2)
         return f(X, Y)
+
     elif isinstance(arg1, DataFrame):
+        def dataframe_from_int_dict(data, frame_template):
+            result = DataFrame(data, index=frame_template.index)
+            if len(result.columns) > 0:
+                result.columns = frame_template.columns[result.columns]
+            return result
+
         results = {}
         if isinstance(arg2, DataFrame):
-            X, Y = arg1.align(arg2, join='outer')
             if pairwise is False:
-                X = X + 0 * Y
-                Y = Y + 0 * X
-                res_columns = arg1.columns.union(arg2.columns)
-                for col in res_columns:
-                    if col in X and col in Y:
-                        results[col] = f(X[col], Y[col])
+                if arg1 is arg2:
+                    # special case in order to handle duplicate column names
+                    for i, col in enumerate(arg1.columns):
+                        results[i] = f(arg1.iloc[:, i], arg2.iloc[:, i])
+                    return dataframe_from_int_dict(results, arg1)
+                else:
+                    if not arg1.columns.is_unique:
+                        raise ValueError("'arg1' columns are not unique")
+                    if not arg2.columns.is_unique:
+                        raise ValueError("'arg2' columns are not unique")
+                    X, Y = arg1.align(arg2, join='outer')
+                    X = X + 0 * Y
+                    Y = Y + 0 * X
+                    res_columns = arg1.columns.union(arg2.columns)
+                    for col in res_columns:
+                        if col in X and col in Y:
+                            results[col] = f(X[col], Y[col])
+                    return DataFrame(results, index=X.index, columns=res_columns)
             elif pairwise is True:
                 results = defaultdict(dict)
                 for i, k1 in enumerate(arg1.columns):
                     for j, k2 in enumerate(arg2.columns):
                         if j<i and arg2 is arg1:
                             # Symmetric case
-                            results[k1][k2] = results[k2][k1]
+                            results[i][j] = results[j][i]
                         else:
-                            results[k1][k2] = f(*_prep_binary(arg1[k1], arg2[k2]))
-                return Panel.from_dict(results).swapaxes('items', 'major')
+                            results[i][j] = f(*_prep_binary(arg1.iloc[:, i], arg2.iloc[:, j]))
+                p = Panel.from_dict(results).swapaxes('items', 'major')
+                if len(p.major_axis) > 0:
+                    p.major_axis = arg1.columns[p.major_axis]
+                if len(p.minor_axis) > 0:
+                    p.minor_axis = arg2.columns[p.minor_axis]
+                return p
             else:
                 raise ValueError("'pairwise' is not True/False")
         else:
-            res_columns = arg1.columns
-            X, Y = arg1.align(arg2, axis=0, join='outer')
             results = {}
+            for i, col in enumerate(arg1.columns):
+                results[i] = f(*_prep_binary(arg1.iloc[:, i], arg2))
+            return dataframe_from_int_dict(results, arg1)
 
-            for col in res_columns:
-                results[col] = f(X[col], Y)
-
-        return DataFrame(results, index=X.index, columns=res_columns)
     else:
         return _flex_binary_moment(arg2, arg1, f)
 
@@ -301,7 +347,8 @@ def _flex_binary_moment(arg1, arg2, f, pairwise=False):
 def rolling_corr_pairwise(df1, df2=None, window=None, min_periods=None,
                           freq=None, center=False):
     import warnings
-    warnings.warn("rolling_corr_pairwise is deprecated, use rolling_corr(..., pairwise=True)", FutureWarning)
+    msg = "rolling_corr_pairwise is deprecated, use rolling_corr(..., pairwise=True)"
+    warnings.warn(msg, FutureWarning, stacklevel=2)
     return rolling_corr(df1, df2, window=window, min_periods=min_periods,
                         freq=freq, center=center,
                         pairwise=True)
@@ -337,19 +384,27 @@ def _rolling_moment(arg, window, func, minp, axis=0, freq=None, center=False,
     y : type of input
     """
     arg = _conv_timerule(arg, freq, how)
-    calc = lambda x: func(x, window, minp=minp, args=args, kwargs=kwargs,
-                          **kwds)
-    return_hook, values = _process_data_structure(arg)
-    # actually calculate the moment. Faster way to do this?
-    if values.ndim > 1:
-        result = np.apply_along_axis(calc, axis, values)
-    else:
-        result = calc(values)
 
-    rs = return_hook(result)
+    return_hook, values = _process_data_structure(arg)
+
+    if values.size == 0:
+        result = values.copy()
+    else:
+        # actually calculate the moment. Faster way to do this?
+        offset = int((window - 1) / 2.) if center else 0
+        additional_nans = np.array([np.NaN] * offset)
+        calc = lambda x: func(np.concatenate((x, additional_nans)) if center else x,
+                              window, minp=minp, args=args, kwargs=kwargs,
+                              **kwds)
+        if values.ndim > 1:
+            result = np.apply_along_axis(calc, axis, values)
+        else:
+            result = calc(values)
+
     if center:
-        rs = _center_window(rs, window, axis)
-    return rs
+        result = _center_window(result, window, axis)
+
+    return return_hook(result)
 
 
 def _center_window(rs, window, axis):
@@ -358,20 +413,13 @@ def _center_window(rs, window, axis):
                          "dimensions")
 
     offset = int((window - 1) / 2.)
-    if isinstance(rs, (Series, DataFrame, Panel)):
-        rs = rs.shift(-offset, axis=axis)
-    else:
-        rs_indexer = [slice(None)] * rs.ndim
-        rs_indexer[axis] = slice(None, -offset)
-
-        lead_indexer = [slice(None)] * rs.ndim
-        lead_indexer[axis] = slice(offset, None)
-
-        na_indexer = [slice(None)] * rs.ndim
-        na_indexer[axis] = slice(-offset, None)
-
-        rs[tuple(rs_indexer)] = np.copy(rs[tuple(lead_indexer)])
-        rs[tuple(na_indexer)] = np.nan
+    if offset > 0:
+        if isinstance(rs, (Series, DataFrame, Panel)):
+            rs = rs.slice_shift(-offset, axis=axis)
+        else:
+            lead_indexer = [slice(None)] * rs.ndim
+            lead_indexer[axis] = slice(offset, None)
+            rs = np.copy(rs[tuple(lead_indexer)])
     return rs
 
 
@@ -382,7 +430,7 @@ def _process_data_structure(arg, kill_inf=True):
         values = arg.values
     elif isinstance(arg, Series):
         values = arg.values
-        return_hook = lambda v: Series(v, arg.index)
+        return_hook = lambda v: Series(v, arg.index, name=arg.name)
     else:
         return_hook = lambda v: v
         values = arg
@@ -422,49 +470,47 @@ def _get_center_of_mass(com, span, halflife):
               _type_of_input_retval, _ewm_notes)
 @Appender(_doc_template)
 def ewma(arg, com=None, span=None, halflife=None, min_periods=0, freq=None,
-         adjust=True, how=None):
-    com = _get_center_of_mass(com, span, halflife)
+         adjust=True, how=None, ignore_na=False):
     arg = _conv_timerule(arg, freq, how)
+    com = _get_center_of_mass(com, span, halflife)
 
     def _ewma(v):
-        result = algos.ewma(v, com, int(adjust))
-        first_index = _first_valid_index(v)
-        result[first_index: first_index + min_periods] = NaN
-        return result
+        return algos.ewma(v, com, int(adjust), int(ignore_na), int(min_periods))
 
     return_hook, values = _process_data_structure(arg)
-    output = np.apply_along_axis(_ewma, 0, values)
+    if values.size == 0:
+        output = values.copy()
+    else:
+        output = np.apply_along_axis(_ewma, 0, values)
     return return_hook(output)
 
 
-def _first_valid_index(arr):
-    # argmax scans from left
-    return notnull(arr).argmax() if len(arr) else 0
-
-
 @Substitution("Exponentially-weighted moving variance", _unary_arg,
               _ewm_kw+_bias_kw, _type_of_input_retval, _ewm_notes)
 @Appender(_doc_template)
 def ewmvar(arg, com=None, span=None, halflife=None, min_periods=0, bias=False,
-           freq=None, how=None):
-    com = _get_center_of_mass(com, span, halflife)
+           freq=None, how=None, ignore_na=False, adjust=True):
     arg = _conv_timerule(arg, freq, how)
-    moment2nd = ewma(arg * arg, com=com, min_periods=min_periods)
-    moment1st = ewma(arg, com=com, min_periods=min_periods)
+    com = _get_center_of_mass(com, span, halflife)
 
-    result = moment2nd - moment1st ** 2
-    if not bias:
-        result *= (1.0 + 2.0 * com) / (2.0 * com)
+    def _ewmvar(v):
+        return algos.ewmcov(v, v, com, int(adjust), int(ignore_na), int(min_periods), int(bias))
 
-    return result
+    return_hook, values = _process_data_structure(arg)
+    if values.size == 0:
+        output = values.copy()
+    else:
+        output = np.apply_along_axis(_ewmvar, 0, values)
+    return return_hook(output)
 
 
 @Substitution("Exponentially-weighted moving std", _unary_arg,
               _ewm_kw+_bias_kw, _type_of_input_retval, _ewm_notes)
 @Appender(_doc_template)
-def ewmstd(arg, com=None, span=None, halflife=None, min_periods=0, bias=False):
+def ewmstd(arg, com=None, span=None, halflife=None, min_periods=0, bias=False,
+           ignore_na=False, adjust=True):
     result = ewmvar(arg, com=com, span=span, halflife=halflife,
-                    min_periods=min_periods, bias=bias)
+                    min_periods=min_periods, bias=bias, adjust=adjust, ignore_na=ignore_na)
     return _zsqrt(result)
 
 ewmvol = ewmstd
@@ -474,7 +520,7 @@ ewmvol = ewmstd
               _ewm_kw+_pairwise_kw, _type_of_input_retval, _ewm_notes)
 @Appender(_doc_template)
 def ewmcov(arg1, arg2=None, com=None, span=None, halflife=None, min_periods=0,
-           bias=False, freq=None, pairwise=None, how=None):
+           bias=False, freq=None, pairwise=None, how=None, ignore_na=False, adjust=True):
     if arg2 is None:
         arg2 = arg1
         pairwise = True if pairwise is None else pairwise
@@ -484,16 +530,17 @@ def ewmcov(arg1, arg2=None, com=None, span=None, halflife=None, min_periods=0,
         pairwise = True if pairwise is None else pairwise
     arg1 = _conv_timerule(arg1, freq, how)
     arg2 = _conv_timerule(arg2, freq, how)
+    com = _get_center_of_mass(com, span, halflife)
 
     def _get_ewmcov(X, Y):
-        mean = lambda x: ewma(x, com=com, span=span, halflife=halflife, min_periods=min_periods)
-        return (mean(X * Y) - mean(X) * mean(Y))
+        # X and Y have the same structure (and NaNs) when called from _flex_binary_moment()
+        return_hook, x_values = _process_data_structure(X)
+        return_hook, y_values = _process_data_structure(Y)
+        cov = algos.ewmcov(x_values, y_values, com, int(adjust), int(ignore_na), int(min_periods), int(bias))
+        return return_hook(cov)
+
     result = _flex_binary_moment(arg1, arg2, _get_ewmcov,
                                  pairwise=bool(pairwise))
-    if not bias:
-        com = _get_center_of_mass(com, span, halflife)
-        result *= (1.0 + 2.0 * com) / (2.0 * com)
-
     return result
 
 
@@ -501,7 +548,7 @@ def ewmcov(arg1, arg2=None, com=None, span=None, halflife=None, min_periods=0,
               _ewm_kw+_pairwise_kw, _type_of_input_retval, _ewm_notes)
 @Appender(_doc_template)
 def ewmcorr(arg1, arg2=None, com=None, span=None, halflife=None, min_periods=0,
-            freq=None, pairwise=None, how=None):
+            freq=None, pairwise=None, how=None, ignore_na=False, adjust=True):
     if arg2 is None:
         arg2 = arg1
         pairwise = True if pairwise is None else pairwise
@@ -511,12 +558,18 @@ def ewmcorr(arg1, arg2=None, com=None, span=None, halflife=None, min_periods=0,
         pairwise = True if pairwise is None else pairwise
     arg1 = _conv_timerule(arg1, freq, how)
     arg2 = _conv_timerule(arg2, freq, how)
+    com = _get_center_of_mass(com, span, halflife)
 
     def _get_ewmcorr(X, Y):
-        mean = lambda x: ewma(x, com=com, span=span, halflife=halflife, min_periods=min_periods)
-        var = lambda x: ewmvar(x, com=com, span=span, halflife=halflife, min_periods=min_periods,
-                               bias=True)
-        return (mean(X * Y) - mean(X) * mean(Y)) / _zsqrt(var(X) * var(Y))
+        # X and Y have the same structure (and NaNs) when called from _flex_binary_moment()
+        return_hook, x_values = _process_data_structure(X)
+        return_hook, y_values = _process_data_structure(Y)
+        cov = algos.ewmcov(x_values, y_values, com, int(adjust), int(ignore_na), int(min_periods), 1)
+        x_var = algos.ewmcov(x_values, x_values, com, int(adjust), int(ignore_na), int(min_periods), 1)
+        y_var = algos.ewmcov(y_values, y_values, com, int(adjust), int(ignore_na), int(min_periods), 1)
+        corr = cov / _zsqrt(x_var * y_var)
+        return return_hook(corr)
+
     result = _flex_binary_moment(arg1, arg2, _get_ewmcorr,
                                  pairwise=bool(pairwise))
     return result
@@ -576,14 +629,14 @@ def _use_window(minp, window):
         return minp
 
 
-def _rolling_func(func, desc, check_minp=_use_window, how=None):
+def _rolling_func(func, desc, check_minp=_use_window, how=None, additional_kw=''):
     if how is None:
         how_arg_str = 'None'
     else:
         how_arg_str = "'%s"%how
 
-    @Substitution(desc, _unary_arg, _roll_kw%how_arg_str, _type_of_input_retval,
-                  _roll_notes)
+    @Substitution(desc, _unary_arg, _roll_kw%how_arg_str + additional_kw,
+                  _type_of_input_retval, _roll_notes)
     @Appender(_doc_template)
     @wraps(func)
     def f(arg, window, min_periods=None, freq=None, center=False, how=how,
@@ -596,18 +649,20 @@ def _rolling_func(func, desc, check_minp=_use_window, how=None):
 
     return f
 
-rolling_max = _rolling_func(algos.roll_max2, 'Moving maximum.', how='max')
-rolling_min = _rolling_func(algos.roll_min2, 'Moving minimum.', how='min')
+rolling_max = _rolling_func(algos.roll_max, 'Moving maximum.', how='max')
+rolling_min = _rolling_func(algos.roll_min, 'Moving minimum.', how='min')
 rolling_sum = _rolling_func(algos.roll_sum, 'Moving sum.')
 rolling_mean = _rolling_func(algos.roll_mean, 'Moving mean.')
-rolling_median = _rolling_func(algos.roll_median_cython, 'Moving median.',
+rolling_median = _rolling_func(algos.roll_median_c, 'Moving median.',
                                how='median')
 
 _ts_std = lambda *a, **kw: _zsqrt(algos.roll_var(*a, **kw))
-rolling_std = _rolling_func(_ts_std, 'Unbiased moving standard deviation.',
-                            check_minp=_require_min_periods(1))
-rolling_var = _rolling_func(algos.roll_var, 'Unbiased moving variance.',
-                            check_minp=_require_min_periods(1))
+rolling_std = _rolling_func(_ts_std, 'Moving standard deviation.',
+                            check_minp=_require_min_periods(1),
+                            additional_kw=_ddof_kw)
+rolling_var = _rolling_func(algos.roll_var, 'Moving variance.',
+                            check_minp=_require_min_periods(1),
+                            additional_kw=_ddof_kw)
 rolling_skew = _rolling_func(algos.roll_skew, 'Unbiased moving skewness.',
                              check_minp=_require_min_periods(3))
 rolling_kurt = _rolling_func(algos.roll_kurt, 'Unbiased moving kurtosis.',
@@ -694,11 +749,12 @@ def rolling_apply(arg, window, func, min_periods=None, freq=None,
     frequency by resampling the data. This is done with the default parameters
     of :meth:`~pandas.Series.resample` (i.e. using the `mean`).
     """
+    offset = int((window - 1) / 2.) if center else 0
     def call_cython(arg, window, minp, args, kwargs):
         minp = _use_window(minp, window)
-        return algos.roll_generic(arg, window, minp, func, args, kwargs)
+        return algos.roll_generic(arg, window, minp, offset, func, args, kwargs)
     return _rolling_moment(arg, window, call_cython, min_periods, freq=freq,
-                           center=center, args=args, kwargs=kwargs)
+                           center=False, args=args, kwargs=kwargs)
 
 
 def rolling_window(arg, window=None, win_type=None, min_periods=None,
@@ -782,13 +838,19 @@ def rolling_window(arg, window=None, win_type=None, min_periods=None,
     arg = _conv_timerule(arg, freq, how)
     return_hook, values = _process_data_structure(arg)
 
-    f = lambda x: algos.roll_window(x, window, minp, avg=mean)
-    result = np.apply_along_axis(f, axis, values)
+    if values.size == 0:
+        result = values.copy()
+    else:
+        offset = int((len(window) - 1) / 2.) if center else 0
+        additional_nans = np.array([np.NaN] * offset)
+        f = lambda x: algos.roll_window(np.concatenate((x, additional_nans)) if center else x,
+                                        window, minp, avg=mean)
+        result = np.apply_along_axis(f, axis, values)
 
-    rs = return_hook(result)
     if center:
-        rs = _center_window(rs, len(window), axis)
-    return rs
+        result = _center_window(result, len(window), axis)
+
+    return return_hook(result)
 
 
 def _validate_win_type(win_type, kwargs):
@@ -813,42 +875,41 @@ def _pop_args(win_type, arg_names, kwargs):
     return all_args
 
 
-def _expanding_func(func, desc, check_minp=_use_window):
-    @Substitution(desc, _unary_arg, _expanding_kw, _type_of_input_retval, "")
+def _expanding_func(func, desc, check_minp=_use_window, additional_kw=''):
+    @Substitution(desc, _unary_arg, _expanding_kw + additional_kw,
+                  _type_of_input_retval, "")
     @Appender(_doc_template)
     @wraps(func)
-    def f(arg, min_periods=1, freq=None, center=False, **kwargs):
-        window = len(arg)
+    def f(arg, min_periods=1, freq=None, **kwargs):
+        window = max(len(arg), min_periods) if min_periods else len(arg)
 
         def call_cython(arg, window, minp, args=(), kwargs={}, **kwds):
             minp = check_minp(minp, window)
             return func(arg, window, minp, **kwds)
         return _rolling_moment(arg, window, call_cython, min_periods, freq=freq,
-                               center=center, **kwargs)
+                               **kwargs)
 
     return f
 
-expanding_max = _expanding_func(algos.roll_max2, 'Expanding maximum.')
-expanding_min = _expanding_func(algos.roll_min2, 'Expanding minimum.')
+expanding_max = _expanding_func(algos.roll_max, 'Expanding maximum.')
+expanding_min = _expanding_func(algos.roll_min, 'Expanding minimum.')
 expanding_sum = _expanding_func(algos.roll_sum, 'Expanding sum.')
 expanding_mean = _expanding_func(algos.roll_mean, 'Expanding mean.')
-expanding_median = _expanding_func(
-    algos.roll_median_cython, 'Expanding median.')
+expanding_median = _expanding_func(algos.roll_median_c, 'Expanding median.')
 
-expanding_std = _expanding_func(_ts_std,
-                                'Unbiased expanding standard deviation.',
-                                check_minp=_require_min_periods(2))
-expanding_var = _expanding_func(algos.roll_var, 'Unbiased expanding variance.',
-                                check_minp=_require_min_periods(2))
-expanding_skew = _expanding_func(
-    algos.roll_skew, 'Unbiased expanding skewness.',
-    check_minp=_require_min_periods(3))
-expanding_kurt = _expanding_func(
-    algos.roll_kurt, 'Unbiased expanding kurtosis.',
-    check_minp=_require_min_periods(4))
+expanding_std = _expanding_func(_ts_std, 'Expanding standard deviation.',
+                                check_minp=_require_min_periods(1),
+                                additional_kw=_ddof_kw)
+expanding_var = _expanding_func(algos.roll_var, 'Expanding variance.',
+                                check_minp=_require_min_periods(1),
+                                additional_kw=_ddof_kw)
+expanding_skew = _expanding_func(algos.roll_skew, 'Unbiased expanding skewness.',
+                                 check_minp=_require_min_periods(3))
+expanding_kurt = _expanding_func(algos.roll_kurt, 'Unbiased expanding kurtosis.',
+                                 check_minp=_require_min_periods(4))
 
 
-def expanding_count(arg, freq=None, center=False):
+def expanding_count(arg, freq=None):
     """
     Expanding count of number of non-NaN observations.
 
@@ -858,8 +919,6 @@ def expanding_count(arg, freq=None, center=False):
     freq : string or DateOffset object, optional (default None)
         Frequency to conform the data to before computing the statistic. Specified
         as a frequency string or DateOffset object.
-    center : boolean, default False
-        Whether the label should correspond with center of window.
 
     Returns
     -------
@@ -871,11 +930,10 @@ def expanding_count(arg, freq=None, center=False):
     frequency by resampling the data. This is done with the default parameters
     of :meth:`~pandas.Series.resample` (i.e. using the `mean`).
     """
-    return rolling_count(arg, len(arg), freq=freq, center=center)
+    return rolling_count(arg, len(arg), freq=freq)
 
 
-def expanding_quantile(arg, quantile, min_periods=1, freq=None,
-                       center=False):
+def expanding_quantile(arg, quantile, min_periods=1, freq=None):
     """Expanding quantile.
 
     Parameters
@@ -889,8 +947,6 @@ def expanding_quantile(arg, quantile, min_periods=1, freq=None,
     freq : string or DateOffset object, optional (default None)
         Frequency to conform the data to before computing the statistic. Specified
         as a frequency string or DateOffset object.
-    center : boolean, default False
-        Whether the label should correspond with center of window.
 
     Returns
     -------
@@ -903,14 +959,13 @@ def expanding_quantile(arg, quantile, min_periods=1, freq=None,
     of :meth:`~pandas.Series.resample` (i.e. using the `mean`).
     """
     return rolling_quantile(arg, len(arg), quantile, min_periods=min_periods,
-                            freq=freq, center=center)
+                            freq=freq)
 
 
 @Substitution("Unbiased expanding covariance.", _binary_arg_flex,
-              _expanding_kw+_pairwise_kw, _flex_retval, "")
+              _expanding_kw+_pairwise_kw+_ddof_kw, _flex_retval, "")
 @Appender(_doc_template)
-def expanding_cov(arg1, arg2=None, min_periods=1, freq=None, center=False,
-                  pairwise=None):
+def expanding_cov(arg1, arg2=None, min_periods=1, freq=None, pairwise=None, ddof=1):
     if arg2 is None:
         arg2 = arg1
         pairwise = True if pairwise is None else pairwise
@@ -918,17 +973,16 @@ def expanding_cov(arg1, arg2=None, min_periods=1, freq=None, center=False,
         min_periods = arg2
         arg2 = arg1
         pairwise = True if pairwise is None else pairwise
-    window = len(arg1) + len(arg2)
+    window = max((len(arg1) + len(arg2)), min_periods) if min_periods else (len(arg1) + len(arg2))
     return rolling_cov(arg1, arg2, window,
                        min_periods=min_periods, freq=freq,
-                       center=center, pairwise=pairwise)
+                       pairwise=pairwise, ddof=ddof)
 
 
 @Substitution("Expanding sample correlation.", _binary_arg_flex,
               _expanding_kw+_pairwise_kw, _flex_retval, "")
 @Appender(_doc_template)
-def expanding_corr(arg1, arg2=None, min_periods=1, freq=None, center=False,
-                   pairwise=None):
+def expanding_corr(arg1, arg2=None, min_periods=1, freq=None, pairwise=None):
     if arg2 is None:
         arg2 = arg1
         pairwise = True if pairwise is None else pairwise
@@ -936,25 +990,25 @@ def expanding_corr(arg1, arg2=None, min_periods=1, freq=None, center=False,
         min_periods = arg2
         arg2 = arg1
         pairwise = True if pairwise is None else pairwise
-    window = len(arg1) + len(arg2)
+    window = max((len(arg1) + len(arg2)), min_periods) if min_periods else (len(arg1) + len(arg2))
     return rolling_corr(arg1, arg2, window,
                         min_periods=min_periods,
-                        freq=freq, center=center, pairwise=pairwise)
+                        freq=freq, pairwise=pairwise)
 
 
 @Substitution("Deprecated. Use expanding_corr(..., pairwise=True) instead.\n\n"
               "Pairwise expanding sample correlation", _pairwise_arg,
               _expanding_kw, _pairwise_retval, "")
 @Appender(_doc_template)
-def expanding_corr_pairwise(df1, df2=None, min_periods=1, freq=None,
-                            center=False):
+def expanding_corr_pairwise(df1, df2=None, min_periods=1, freq=None):
     import warnings
-    warnings.warn("expanding_corr_pairwise is deprecated, use expanding_corr(..., pairwise=True)", FutureWarning)
+    msg = "expanding_corr_pairwise is deprecated, use expanding_corr(..., pairwise=True)"
+    warnings.warn(msg, FutureWarning, stacklevel=2)
     return expanding_corr(df1, df2, min_periods=min_periods,
-                          freq=freq, center=center, pairwise=True)
+                          freq=freq, pairwise=True)
 
 
-def expanding_apply(arg, func, min_periods=1, freq=None, center=False,
+def expanding_apply(arg, func, min_periods=1, freq=None,
                     args=(), kwargs={}):
     """Generic expanding function application.
 
@@ -969,8 +1023,6 @@ def expanding_apply(arg, func, min_periods=1, freq=None, center=False,
     freq : string or DateOffset object, optional (default None)
         Frequency to conform the data to before computing the statistic. Specified
         as a frequency string or DateOffset object.
-    center : boolean, default False
-        Whether the label should correspond with center of window.
     args : tuple
         Passed on to func
     kwargs : dict
@@ -986,6 +1038,6 @@ def expanding_apply(arg, func, min_periods=1, freq=None, center=False,
     frequency by resampling the data. This is done with the default parameters
     of :meth:`~pandas.Series.resample` (i.e. using the `mean`).
     """
-    window = len(arg)
+    window = max(len(arg), min_periods) if min_periods else len(arg)
     return rolling_apply(arg, window, func, min_periods=min_periods, freq=freq,
-                         center=center, args=args, kwargs=kwargs)
+                         args=args, kwargs=kwargs)
