@@ -1,12 +1,19 @@
+# -*- coding: utf-8 -*-
 # pylint: disable=W0102
 
+from datetime import datetime, date
+
 import nose
 import numpy as np
 
-from pandas import Index, MultiIndex, DataFrame, Series, Categorical
+import re
+import itertools
+from pandas import Index, MultiIndex, DataFrame, DatetimeIndex, Series, Categorical
 from pandas.compat import OrderedDict, lrange
 from pandas.sparse.array import SparseArray
-from pandas.core.internals import *
+from pandas.core.internals import (BlockPlacement, SingleBlockManager, make_block,
+                                   BlockManager)
+import pandas.core.common as com
 import pandas.core.internals as internals
 import pandas.util.testing as tm
 import pandas as pd
@@ -41,7 +48,7 @@ def create_block(typestr, placement, item_shape=None, num_offset=0):
         * complex, c16, c8
         * bool
         * object, string, O
-        * datetime, dt, M8[ns]
+        * datetime, dt, M8[ns], M8[ns, tz]
         * timedelta, td, m8[ns]
         * sparse (SparseArray with fill_value=0.0)
         * sparse_na (SparseArray with fill_value=np.nan)
@@ -67,15 +74,22 @@ def create_block(typestr, placement, item_shape=None, num_offset=0):
     elif typestr in ('object', 'string', 'O'):
         values = np.reshape(['A%d' % i for i in mat.ravel() + num_offset],
                             shape)
-    elif typestr in ('bool'):
+    elif typestr in ('b','bool',):
         values = np.ones(shape, dtype=np.bool_)
     elif typestr in ('datetime', 'dt', 'M8[ns]'):
         values = (mat * 1e9).astype('M8[ns]')
+    elif typestr.startswith('M8[ns'):
+        # datetime with tz
+        m = re.search('M8\[ns,\s*(\w+\/?\w*)\]', typestr)
+        assert m is not None, "incompatible typestr -> {0}".format(typestr)
+        tz = m.groups()[0]
+        assert num_items == 1, "must have only 1 num items for a tz-aware"
+        values = DatetimeIndex(np.arange(N) * 1e9, tz=tz)
     elif typestr in ('timedelta', 'td', 'm8[ns]'):
         values = (mat * 1).astype('m8[ns]')
-    elif typestr in ('category'):
+    elif typestr in ('category',):
         values = Categorical([1,1,2,2,3,3,3,3,4,4])
-    elif typestr in ('category2'):
+    elif typestr in ('category2',):
         values = Categorical(['a','a','a','a','b','b','c','c','c','d'])
     elif typestr in ('sparse', 'sparse_na'):
         # FIXME: doesn't support num_rows != 10
@@ -285,6 +299,26 @@ class TestBlock(tm.TestCase):
         pass
 
 
+class TestDatetimeBlock(tm.TestCase):
+    _multiprocess_can_split_ = True
+
+    def test_try_coerce_arg(self):
+        block = create_block('datetime', [0])
+
+        # coerce None
+        none_coerced = block._try_coerce_args(block.values, None)[2]
+        self.assertTrue(pd.Timestamp(none_coerced) is pd.NaT)
+
+        # coerce different types of date bojects
+        vals = (np.datetime64('2010-10-10'),
+                datetime(2010, 10, 10),
+                date(2010, 10, 10))
+        for val in vals:
+            coerced = block._try_coerce_args(block.values, val)[2]
+            self.assertEqual(np.int64, type(coerced))
+            self.assertEqual(pd.Timestamp('2010-10-10'), pd.Timestamp(coerced))
+
+
 class TestBlockManager(tm.TestCase):
     _multiprocess_can_split_ = True
 
@@ -378,7 +412,7 @@ class TestBlockManager(tm.TestCase):
                 res = self.mgr.get_scalar((item, index))
                 exp = self.mgr.get(item, fastpath=False)[i]
                 assert_almost_equal(res, exp)
-                exp = self.mgr.get(item).values[i]
+                exp = self.mgr.get(item).internal_values()[i]
                 assert_almost_equal(res, exp)
 
     def test_get(self):
@@ -391,19 +425,19 @@ class TestBlockManager(tm.TestCase):
         assert_almost_equal(mgr.get('a', fastpath=False), values[0])
         assert_almost_equal(mgr.get('b', fastpath=False), values[1])
         assert_almost_equal(mgr.get('c', fastpath=False), values[2])
-        assert_almost_equal(mgr.get('a').values, values[0])
-        assert_almost_equal(mgr.get('b').values, values[1])
-        assert_almost_equal(mgr.get('c').values, values[2])
+        assert_almost_equal(mgr.get('a').internal_values(), values[0])
+        assert_almost_equal(mgr.get('b').internal_values(), values[1])
+        assert_almost_equal(mgr.get('c').internal_values(), values[2])
 
     def test_set(self):
         mgr = create_mgr('a,b,c: int', item_shape=(3,))
 
         mgr.set('d', np.array(['foo'] * 3))
         mgr.set('b', np.array(['bar'] * 3))
-        assert_almost_equal(mgr.get('a').values, [0] * 3)
-        assert_almost_equal(mgr.get('b').values, ['bar'] * 3)
-        assert_almost_equal(mgr.get('c').values, [2] * 3)
-        assert_almost_equal(mgr.get('d').values, ['foo'] * 3)
+        assert_almost_equal(mgr.get('a').internal_values(), [0] * 3)
+        assert_almost_equal(mgr.get('b').internal_values(), ['bar'] * 3)
+        assert_almost_equal(mgr.get('c').internal_values(), [2] * 3)
+        assert_almost_equal(mgr.get('d').internal_values(), ['foo'] * 3)
 
     def test_insert(self):
         self.mgr.insert(0, 'inserted', np.arange(N))
@@ -430,6 +464,17 @@ class TestBlockManager(tm.TestCase):
         mgr2.set('quux', randn(N))
         self.assertEqual(mgr2.get('quux').dtype, np.float_)
 
+    def test_set_change_dtype_slice(self): # GH8850
+        cols = MultiIndex.from_tuples([('1st','a'), ('2nd','b'), ('3rd','c')])
+        df = DataFrame([[1.0, 2, 3], [4.0, 5, 6]], columns=cols)
+        df['2nd'] = df['2nd'] * 2.0
+
+        self.assertEqual(sorted(df.blocks.keys()), ['float64', 'int64'])
+        assert_frame_equal(df.blocks['float64'],
+                DataFrame([[1.0, 4.0], [4.0, 10.0]], columns=cols[:2]))
+        assert_frame_equal(df.blocks['int64'],
+                DataFrame([[3], [6]], columns=cols[2:]))
+
     def test_copy(self):
         shallow = self.mgr.copy(deep=False)
 
@@ -444,7 +489,6 @@ class TestBlockManager(tm.TestCase):
 
     def test_sparse(self):
         mgr = create_mgr('a: sparse-1; b: sparse-2')
-
         # what to test here?
         self.assertEqual(mgr.as_matrix().dtype, np.float64)
 
@@ -476,6 +520,12 @@ class TestBlockManager(tm.TestCase):
         mgr = create_mgr('h: datetime-1; g: datetime-2')
         self.assertEqual(mgr.as_matrix().dtype, 'M8[ns]')
 
+    def test_as_matrix_datetime_tz(self):
+        mgr = create_mgr('h: M8[ns, US/Eastern]; g: M8[ns, CET]')
+        self.assertEqual(mgr.get('h').dtype, 'datetime64[ns, US/Eastern]')
+        self.assertEqual(mgr.get('g').dtype, 'datetime64[ns, CET]')
+        self.assertEqual(mgr.as_matrix().dtype, 'object')
+
     def test_astype(self):
         # coerce all
         mgr = create_mgr('c: f4; d: f2; e: f8')
@@ -542,7 +592,7 @@ class TestBlockManager(tm.TestCase):
         mgr.set('a', np.array(['1'] * N, dtype=np.object_))
         mgr.set('b', np.array(['2.'] * N, dtype=np.object_))
         mgr.set('foo', np.array(['foo.'] * N, dtype=np.object_))
-        new_mgr = mgr.convert(convert_numeric=True)
+        new_mgr = mgr.convert(numeric=True)
         self.assertEqual(new_mgr.get('a').dtype, np.int64)
         self.assertEqual(new_mgr.get('b').dtype, np.float64)
         self.assertEqual(new_mgr.get('foo').dtype, np.object_)
@@ -554,7 +604,7 @@ class TestBlockManager(tm.TestCase):
         mgr.set('a', np.array(['1'] * N, dtype=np.object_))
         mgr.set('b', np.array(['2.'] * N, dtype=np.object_))
         mgr.set('foo', np.array(['foo.'] * N, dtype=np.object_))
-        new_mgr = mgr.convert(convert_numeric=True)
+        new_mgr = mgr.convert(numeric=True)
         self.assertEqual(new_mgr.get('a').dtype, np.int64)
         self.assertEqual(new_mgr.get('b').dtype, np.float64)
         self.assertEqual(new_mgr.get('foo').dtype, np.object_)
@@ -617,14 +667,16 @@ class TestBlockManager(tm.TestCase):
 
     def test_interleave_non_unique_cols(self):
         df = DataFrame([
-            [Timestamp('20130101'), 3.5],
-            [Timestamp('20130102'), 4.5]],
+            [pd.Timestamp('20130101'), 3.5],
+            [pd.Timestamp('20130102'), 4.5]],
             columns=['x', 'x'],
             index=[1, 2])
 
         df_unique = df.copy()
         df_unique.columns = ['x', 'y']
-        np.testing.assert_array_equal(df_unique.values, df.values)
+        self.assertEqual(df_unique.values.shape, df.values.shape)
+        tm.assert_numpy_array_equal(df_unique.values[0], df.values[0])
+        tm.assert_numpy_array_equal(df_unique.values[1], df.values[1])
 
     def test_consolidate(self):
         pass
@@ -656,10 +708,10 @@ class TestBlockManager(tm.TestCase):
         assert_almost_equal(mgr.get('c',fastpath=False), reindexed.get('c',fastpath=False))
         assert_almost_equal(mgr.get('a',fastpath=False), reindexed.get('a',fastpath=False))
         assert_almost_equal(mgr.get('d',fastpath=False), reindexed.get('d',fastpath=False))
-        assert_almost_equal(mgr.get('g').values, reindexed.get('g').values)
-        assert_almost_equal(mgr.get('c').values, reindexed.get('c').values)
-        assert_almost_equal(mgr.get('a').values, reindexed.get('a').values)
-        assert_almost_equal(mgr.get('d').values, reindexed.get('d').values)
+        assert_almost_equal(mgr.get('g').internal_values(), reindexed.get('g').internal_values())
+        assert_almost_equal(mgr.get('c').internal_values(), reindexed.get('c').internal_values())
+        assert_almost_equal(mgr.get('a').internal_values(), reindexed.get('a').internal_values())
+        assert_almost_equal(mgr.get('d').internal_values(), reindexed.get('d').internal_values())
 
     def test_multiindex_xs(self):
         mgr = create_mgr('a,b,c: f8; d,e,f: i8')
@@ -685,18 +737,18 @@ class TestBlockManager(tm.TestCase):
         numeric = mgr.get_numeric_data()
         assert_almost_equal(numeric.items, ['int', 'float', 'complex', 'bool'])
         assert_almost_equal(mgr.get('float',fastpath=False), numeric.get('float',fastpath=False))
-        assert_almost_equal(mgr.get('float').values, numeric.get('float').values)
+        assert_almost_equal(mgr.get('float').internal_values(), numeric.get('float').internal_values())
 
         # Check sharing
         numeric.set('float', np.array([100., 200., 300.]))
         assert_almost_equal(mgr.get('float',fastpath=False), np.array([100., 200., 300.]))
-        assert_almost_equal(mgr.get('float').values, np.array([100., 200., 300.]))
+        assert_almost_equal(mgr.get('float').internal_values(), np.array([100., 200., 300.]))
 
         numeric2 = mgr.get_numeric_data(copy=True)
         assert_almost_equal(numeric.items, ['int', 'float', 'complex', 'bool'])
         numeric2.set('float', np.array([1000., 2000., 3000.]))
         assert_almost_equal(mgr.get('float',fastpath=False), np.array([100., 200., 300.]))
-        assert_almost_equal(mgr.get('float').values, np.array([100., 200., 300.]))
+        assert_almost_equal(mgr.get('float').internal_values(), np.array([100., 200., 300.]))
 
     def test_get_bool_data(self):
         mgr = create_mgr('int: int; float: float; complex: complex;'
@@ -707,17 +759,17 @@ class TestBlockManager(tm.TestCase):
         bools = mgr.get_bool_data()
         assert_almost_equal(bools.items, ['bool'])
         assert_almost_equal(mgr.get('bool',fastpath=False), bools.get('bool',fastpath=False))
-        assert_almost_equal(mgr.get('bool').values, bools.get('bool').values)
+        assert_almost_equal(mgr.get('bool').internal_values(), bools.get('bool').internal_values())
 
         bools.set('bool', np.array([True, False, True]))
         assert_almost_equal(mgr.get('bool',fastpath=False), [True, False, True])
-        assert_almost_equal(mgr.get('bool').values, [True, False, True])
+        assert_almost_equal(mgr.get('bool').internal_values(), [True, False, True])
 
         # Check sharing
         bools2 = mgr.get_bool_data(copy=True)
         bools2.set('bool', np.array([False, True, False]))
         assert_almost_equal(mgr.get('bool',fastpath=False), [True, False, True])
-        assert_almost_equal(mgr.get('bool').values, [True, False, True])
+        assert_almost_equal(mgr.get('bool').internal_values(), [True, False, True])
 
     def test_unicode_repr_doesnt_raise(self):
         str_repr = repr(create_mgr(u('b,\u05d0: object')))
@@ -739,6 +791,25 @@ class TestBlockManager(tm.TestCase):
         bm2 = BlockManager(bm1.blocks[::-1], bm1.axes)
         self.assertTrue(bm1.equals(bm2))
 
+    def test_equals_block_order_different_dtypes(self):
+        # GH 9330
+
+        mgr_strings = [
+            "a:i8;b:f8", # basic case
+            "a:i8;b:f8;c:c8;d:b", # many types
+            "a:i8;e:dt;f:td;g:string", # more types
+            "a:i8;b:category;c:category2;d:category2", # categories
+            "c:sparse;d:sparse_na;b:f8", # sparse
+            ]
+
+        for mgr_string in mgr_strings:
+            bm = create_mgr(mgr_string)
+            block_perms = itertools.permutations(bm.blocks)
+            for bm_perm in block_perms:
+                bm_this = BlockManager(bm_perm, bm.axes)
+                self.assertTrue(bm.equals(bm_this))
+                self.assertTrue(bm_this.equals(bm))
+
     def test_single_mgr_ctor(self):
         mgr = create_single_mgr('f8', num_rows=5)
         self.assertEqual(mgr.as_matrix().tolist(), [0., 1., 2., 3., 4.])
@@ -781,6 +852,13 @@ class TestIndexing(object):
         def assert_slice_ok(mgr, axis, slobj):
             # import pudb; pudb.set_trace()
             mat = mgr.as_matrix()
+
+            # we maybe using an ndarray to test slicing and
+            # might not be the full length of the axis
+            if isinstance(slobj, np.ndarray):
+                ax = mgr.axes[axis]
+                if len(ax) and len(slobj) and len(slobj) != len(ax):
+                    slobj = np.concatenate([slobj, np.zeros(len(ax)-len(slobj),dtype=bool)])
             sliced = mgr.get_slice(slobj, axis=axis)
             mat_slobj = (slice(None),) * axis + (slobj,)
             assert_almost_equal(mat[mat_slobj], sliced.as_matrix())
@@ -1006,7 +1084,7 @@ class TestBlockPlacement(tm.TestCase):
 
     def test_slice_to_array_conversion(self):
         def assert_as_array_equals(slc, asarray):
-            np.testing.assert_array_equal(
+            tm.assert_numpy_array_equal(
                 BlockPlacement(slc).as_array,
                 np.asarray(asarray))
 
