@@ -1,18 +1,19 @@
 """Tests for cases where the user seeks to obtain packed msgpack objects"""
 
-from pandas import compat
+import io
 from pandas.msgpack import Unpacker, packb
 
+
 def test_write_bytes():
     unpacker = Unpacker()
     unpacker.feed(b'abc')
-    f = compat.BytesIO()
+    f = io.BytesIO()
     assert unpacker.unpack(f.write) == ord('a')
     assert f.getvalue() == b'a'
-    f = compat.BytesIO()
+    f = io.BytesIO()
     assert unpacker.skip(f.write) is None
     assert f.getvalue() == b'b'
-    f = compat.BytesIO()
+    f = io.BytesIO()
     assert unpacker.skip() is None
     assert f.getvalue() == b''
 
@@ -20,9 +21,9 @@ def test_write_bytes():
 def test_write_bytes_multi_buffer():
     long_val = (5) * 100
     expected = packb(long_val)
-    unpacker = Unpacker(compat.BytesIO(expected), read_size=3, max_buffer_size=3)
+    unpacker = Unpacker(io.BytesIO(expected), read_size=3, max_buffer_size=3)
 
-    f = compat.BytesIO()
+    f = io.BytesIO()
     unpacked = unpacker.unpack(f.write)
     assert unpacked == long_val
     assert f.getvalue() == expected
