@@ -1,12 +1,11 @@
-
-import warnings
-
+# -*- coding: utf-8 -*-
 import nose
 
-import pandas.util
 from pandas.util.decorators import deprecate_kwarg
 import pandas.util.testing as tm
 
+
+
 class TestDecorators(tm.TestCase):
     def setUp(self):
         @deprecate_kwarg('old', 'new')
@@ -74,7 +73,6 @@ def test_rands_array():
     assert(arr.shape == (10, 10))
     assert(len(arr[1, 1]) == 7)
 
-
 if __name__ == '__main__':
     nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb', '--pdb-failure'],
                    exit=False)
