@@ -6,7 +6,7 @@ i, j, k = 7, 771, 5532
 
 panel = Panel(np.random.randn(i, j, k),
               items=[rands(10) for _ in range(i)],
-              major_axis=DateRange('1/1/2000', periods=j,
+              major_axis=DatetimeIndex('1/1/2000', periods=j,
                                    offset=datetools.Minute()),
               minor_axis=[rands(10) for _ in range(k)])
 
