@@ -5,7 +5,7 @@ try:
 except:
     import pandas.core.datetools as offsets
 
-common_setup = """from pandas_vb_common import *
+common_setup = """from .pandas_vb_common import *
 try:
     from pandas.tseries.offsets import *
 except:
@@ -17,8 +17,8 @@ except:
 
 setup = common_setup + """
 N, K = 5000, 50
-index = [rands(10) for _ in xrange(N)]
-columns = [rands(10) for _ in xrange(K)]
+index = tm.makeStringIndex(N)
+columns = tm.makeStringIndex(K)
 frame = DataFrame(np.random.randn(N, K), index=index, columns=columns)
 
 try:
@@ -40,7 +40,7 @@ series_ctor_from_dict = Benchmark("Series(some_dict)", setup)
 
 # nested dict, integer indexes, regression described in #621
 setup = common_setup + """
-data = dict((i,dict((j,float(j)) for j in xrange(100))) for i in xrange(2000))
+data = dict((i,dict((j,float(j)) for j in range(100))) for i in xrange(2000))
 """
 frame_ctor_nested_dict_int64 = Benchmark("DataFrame(data)", setup)
 
@@ -50,9 +50,30 @@ frame_ctor_nested_dict_int64 = Benchmark("DataFrame(data)", setup)
 # offset times 1000 can easily go out of Timestamp bounds and raise errors.
 dynamic_benchmarks = {}
 n_steps = [1, 2]
+offset_kwargs = {'WeekOfMonth': {'weekday': 1, 'week': 1},
+                 'LastWeekOfMonth': {'weekday': 1, 'week': 1},
+                 'FY5253': {'startingMonth': 1, 'weekday': 1},
+                 'FY5253Quarter': {'qtr_with_extra_week': 1, 'startingMonth': 1, 'weekday': 1}}
+
+offset_extra_cases = {'FY5253': {'variation': ['nearest', 'last']},
+                      'FY5253Quarter': {'variation': ['nearest', 'last']}}
+
 for offset in offsets.__all__:
     for n in n_steps:
-        setup = common_setup + """
+        kwargs = {}
+        if offset in offset_kwargs:
+            kwargs = offset_kwargs[offset]
+
+        if offset in offset_extra_cases:
+            extras = offset_extra_cases[offset]
+        else:
+            extras = {'': ['']}
+
+        for extra_arg in extras:
+            for extra in extras[extra_arg]:
+                if extra:
+                    kwargs[extra_arg] = extra
+                setup = common_setup + """
 
 def get_period_count(start_date, off):
     ten_offsets_in_days = ((start_date + off * 10) - start_date).days
@@ -69,12 +90,14 @@ def get_index_for_offset(off):
                       periods=min(1000, get_period_count(start_date, off)),
                       freq=off)
 
-idx = get_index_for_offset({}({}))
+idx = get_index_for_offset({}({}, **{}))
 df = DataFrame(np.random.randn(len(idx),10), index=idx)
 d = dict([ (col,df[col]) for col in df.columns ])
-""".format(offset, n)
-        key = 'frame_ctor_dtindex_{}x{}'.format(offset, n)
-        dynamic_benchmarks[key] = Benchmark("DataFrame(d)", setup, name=key)
+""".format(offset, n, kwargs)
+                key = 'frame_ctor_dtindex_{}x{}'.format(offset, n)
+                if extra:
+                    key += '__{}_{}'.format(extra_arg, extra)
+                dynamic_benchmarks[key] = Benchmark("DataFrame(d)", setup, name=key)
 
 # Have to stuff them in globals() so vbench detects them
 globals().update(dynamic_benchmarks)
