@@ -1,5 +1,5 @@
 from pandas import *
-from pandas.util.testing import rands
+import pandas as pd
 from datetime import timedelta
 from numpy.random import randn
 from numpy.random import randint
@@ -7,7 +7,12 @@ from numpy.random import permutation
 import pandas.util.testing as tm
 import random
 import numpy as np
+try:
+    from pandas.compat import range
+except ImportError:
+    pass
 
+np.random.seed(1234)
 try:
     import pandas._tseries as lib
 except:
