@@ -1,7 +1,7 @@
 from vbench.benchmark import Benchmark
 from datetime import datetime
 
-common_setup = """from pandas_vb_common import *
+common_setup = """from .pandas_vb_common import *
 """
 
 #----------------------------------------------------------------------
@@ -34,9 +34,8 @@ setup = common_setup + """
 N = 1000
 K = 20
 
-level1 = np.array([tm.rands(10) for _ in xrange(N)], dtype='O').repeat(K)
-level2 = np.tile(np.array([tm.rands(10) for _ in xrange(K)], dtype='O'),
-                 N)
+level1 = tm.makeStringIndex(N).values.repeat(K)
+level2 = np.tile(tm.makeStringIndex(K).values, N)
 index = MultiIndex.from_arrays([level1, level2])
 
 s1 = Series(np.random.randn(N * K), index=index)
@@ -50,6 +49,18 @@ reindex_multi = Benchmark(statement, setup,
 #----------------------------------------------------------------------
 # Pad / backfill
 
+def pad(source_series, target_index):
+    try:
+        source_series.reindex(target_index, method='pad')
+    except:
+        source_series.reindex(target_index, fillMethod='pad')
+
+def backfill(source_series, target_index):
+    try:
+        source_series.reindex(target_index, method='backfill')
+    except:
+        source_series.reindex(target_index, fillMethod='backfill')
+
 setup = common_setup + """
 rng = date_range('1/1/2000', periods=100000, freq=datetools.Minute())
 
@@ -58,23 +69,23 @@ ts2 = ts[::2]
 ts3 = ts2.reindex(ts.index)
 ts4 = ts3.astype('float32')
 
-def pad():
+def pad(source_series, target_index):
     try:
-        ts2.reindex(ts.index, method='pad')
+        source_series.reindex(target_index, method='pad')
     except:
-        ts2.reindex(ts.index, fillMethod='pad')
-def backfill():
+        source_series.reindex(target_index, fillMethod='pad')
+def backfill(source_series, target_index):
     try:
-        ts2.reindex(ts.index, method='backfill')
+        source_series.reindex(target_index, method='backfill')
     except:
-        ts2.reindex(ts.index, fillMethod='backfill')
+        source_series.reindex(target_index, fillMethod='backfill')
 """
 
-statement = "pad()"
+statement = "pad(ts2, ts.index)"
 reindex_daterange_pad = Benchmark(statement, setup,
                                   name="reindex_daterange_pad")
 
-statement = "backfill()"
+statement = "backfill(ts2, ts.index)"
 reindex_daterange_backfill = Benchmark(statement, setup,
                                        name="reindex_daterange_backfill")
 
@@ -125,8 +136,8 @@ setup = common_setup + """
 N = 10000
 K = 10
 
-key1 = np.array([rands(10) for _ in xrange(N)], dtype='O').repeat(K)
-key2 = np.array([rands(10) for _ in xrange(N)], dtype='O').repeat(K)
+key1 = tm.makeStringIndex(N).values.repeat(K)
+key2 = tm.makeStringIndex(N).values.repeat(K)
 
 df = DataFrame({'key1' : key1, 'key2' : key2,
                 'value' : np.random.randn(N * K)})
@@ -166,7 +177,7 @@ frame_drop_dup_na_inplace = Benchmark(statement2, setup,
 
 setup = common_setup + """
 s = Series(np.random.randint(0, 1000, size=10000))
-s2 = Series(np.tile([rands(10) for i in xrange(1000)], 10))
+s2 = Series(np.tile(tm.makeStringIndex(1000).values, 10))
 """
 
 series_drop_duplicates_int = Benchmark('s.drop_duplicates()', setup,
@@ -195,7 +206,7 @@ frame_fillna_many_columns_pad = Benchmark("df.fillna(method='pad')",
 
 setup = common_setup + """
 n = 50000
-indices = Index([rands(10) for _ in xrange(n)])
+indices = tm.makeStringIndex(n)
 
 def sample(values, k):
     from random import shuffle
